from .body_fat_formulas import *

class BodyFatCalculator:
  def __init__(self, gender, age, weight, height, neck, waist, units, hip=None):
    self.gender = gender
    self.age = age
    self.weight = weight
    self.height = height
    self.neck = neck
    self.waist = waist
    self.hip = hip
    self.units = units
    self.results = {}
    
    # Jackson & Pollard Ideal Body Fat Percentages
    self.ideal_fat_women = {20: "17.7%", 25: "18.4%", 30: "19.3%",
                            35: "21.5%", 40: "22.2%", 45: "22.9%",
                            50: "25.2%", 55: "26.3%"}

    self.ideal_fat_men = {20: "8.5%", 25: "10.5%", 30: "12.7%",
                          35: "13.7%", 40: "15.3%", 45: "16.4%",
                          50: "18.9%", 55: "20.9%"}
    
    # The American Council on Exercise Body Fat Categorization
    self.categorization_women = {"10-13%": "Essential Fat", "14-20%": "Athletes",
                                 "21-24%": "Fitness", "25-31%": "Average", "32+%": "Obese"}

    self.categorization_men = {"2-5%": "Essential Fat", "6-13%": "Athletes",
                               "14-17%": "Fitness", "18-25%": "Average", "25+%": "Obese"}

    self.calculate_body_fat()
    self.create_info()

  def calculate_body_fat(self):
    if self.units == "imperial":
      if self.gender == "male":
        bf_navy_USC = male_navy_USC(self.waist, self.neck, self.height)
        bf_BMI_USC = male_BMI_USC(self.weight, self.height, self.age)
      elif self.gender == "female":
        bf_navy_USC = female_navy_USC(self.waist, self.neck, self.height, self.hip)
        bf_BMI_USC = female_BMI_USC(self.weight, self.height, self.age)
      body_fat_mass = fat_mass(bf_navy_USC, self.weight)
      lean_mass = lean_body_mass(self.weight, body_fat_mass)
      
      self.results["Body Fat Navy"] = round(bf_navy_USC, 1)
      self.results["Body Fat BMI"] = round(bf_BMI_USC, 1)
      self.results["Fat Mass"] = round(body_fat_mass, 1)
      self.results["Lean Body Mass"] = round(lean_mass, 1)
    
    elif self.units == "metric":
      if self.gender == "male":
        bf_navy_SI = male_navy_SI(self.waist, self.neck, self.height)
        bf_BMI_SI = male_BMI_SI(self.weight, self.height, self.age)
      elif self.gender == "female":
        bf_navy_SI = female_navy_SI(self.waist, self.neck, self.height, self.hip)
        bf_BMI_SI = female_BMI_SI(self.weight, self.height, self.age)
      
      body_fat_mass = fat_mass(bf_navy_SI, self.weight)
      lean_mass = lean_body_mass(self.weight, body_fat_mass)
      
      self.results["Body Fat Navy"] = round(bf_navy_SI, 1)
      self.results["Body Fat BMI"] = round(bf_BMI_SI, 1)
      self.results["Fat Mass"] = round(body_fat_mass, 1)
      self.results["Lean Body Mass"] = round(lean_mass, 1)

  def create_info(self):
    if self.gender == "male":
      age_group = self.binary_search_helper(self.ideal_fat_men.keys(), self.age)
      self.results["Ideal Body Fat"] = self.ideal_fat_men[age_group]
    elif self.gender == "female":
      age_group = self.binary_search_helper(self.ideal_fat_women.keys(), self.age)
      self.results["Ideal Body Fat"] = self.ideal_fat_women[age_group]
    
    body_fat_categorization = list(self.categorization_men.keys()) if self.gender == "male" else list(self.categorization_women.keys())
    percentages = [percentage.rstrip("%").strip("+").split("-") for percentage in list(body_fat_categorization)]
    for item in percentages:
      if self.gender == "male" and int(self.results["Body Fat Navy"]) >= 25:
        self.results["Body Fat Category"] = self.categorization_men["25+%"]
      elif self.gender == "female" and int(self.results["Body Fat Navy"]) >= 32:
        self.results["Body Fat Category"] = self.categorization_women["32+%"]
      elif int(self.results["Body Fat Navy"]) >= int(item[0]) and int(self.results["Body Fat Navy"]) <= int(item[1]):
        if self.gender == "female":
          self.results["Body Fat Category"] = self.categorization_women["".join(["-".join(item), "%"])]
        elif self.gender == "male":
          self.results["Body Fat Category"] = self.categorization_men["".join(["-".join(item), "%"])]
    
    if self.results["Ideal Body Fat"] == "8.5%":
      diff_ideal_body_fat = round(self.results["Body Fat Navy"]-float(self.results["Ideal Body Fat"][0:3]), 1)
    else:
      diff_ideal_body_fat = round(self.results["Body Fat Navy"]-float(self.results["Ideal Body Fat"][0:4]), 1)
    
    weight_to_ideal_body_fat = round(fat_mass(diff_ideal_body_fat, self.weight), 1)
    self.results["Body Fat To Lose To Reach Ideal"] = weight_to_ideal_body_fat
  
  def binary_search_helper(self, arr, target):
    arr = list(arr)
    low = 0
    high = len(arr)-1
    while low < high:
      mid = (high+low)//2
      if arr[mid] == target: break
      elif arr[mid] > target: high = mid-1
      else: low = mid+1
    mid = (high+low)//2
    return arr[mid]
   
  def get_results(self):
    return self.results



import os
import importlib

db_info = {"host": "fitnesstracker.cc7s2r4sjjv6.eu-west-3.rds.amazonaws.com", "port": 5432,
           "database": "postgres", "user": "admin", "password": os.environ["FT_ADMIN_PASSWORD"]}

db_path = os.path.join(os.path.dirname(importlib.util.find_spec("fitness_tracker").origin), "fitness_tracker.db")



import json
from PyQt5.QtWidgets import QLabel, QWidget, QPushButton, QFormLayout, QComboBox
from PyQt5.QtGui import  QFont
from PyQt5.QtCore import pyqtSignal
from .big_lifts_db import fetch_preferred_lifts, update_preferred_lifts, update_1RM_and_lifts_for_reps

class PreferredLifts(QWidget):
  change_lifts_signal = pyqtSignal(bool)

  def __init__(self):
    super().__init__()
    self.setWindowTitle("Edit Preferred Lifts")
    self.preferred_lifts = json.loads(fetch_preferred_lifts())
    self.setLayout(self.create_panel())
    self.set_preferred_lifts()
  
  def create_panel(self):
    form_layout = QFormLayout()

    horizontal_press_label = QLabel("Horizontal Press")
    self.horizontal_press_dropdown = QComboBox()
    self.horizontal_press_dropdown.addItems(["Bench Press", "Incline Bench Press"])

    floor_pull_label = QLabel("Floor Pull")
    self.floor_pull_dropdown = QComboBox()
    self.floor_pull_dropdown.addItems(["Deadlift", "Sumo Deadlift"])

    squat_label = QLabel("Squat")
    self.squat_dropdown = QComboBox()
    self.squat_dropdown.addItems(["Back Squat", "Front Squat"])

    vertical_press_label = QLabel("Vertical Press")
    self.vertical_press_dropdown = QComboBox()
    self.vertical_press_dropdown.addItems(["Overhead Press", "Push Press"])
    
    save_button = QPushButton("Save")
    save_button.clicked.connect(lambda: self.save_preferred_lifts())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close())

    form_layout.addRow(horizontal_press_label, self.horizontal_press_dropdown)
    form_layout.addRow(floor_pull_label, self.floor_pull_dropdown)
    form_layout.addRow(squat_label, self.squat_dropdown)
    form_layout.addRow(vertical_press_label, self.vertical_press_dropdown)
    form_layout.addRow(save_button, cancel_button)
    return form_layout

  def set_preferred_lifts(self):
    horizontal_press_index = self.horizontal_press_dropdown.findText(self.preferred_lifts["Horizontal Press"])
    floor_pull_index = self.floor_pull_dropdown.findText(self.preferred_lifts["Floor Pull"])
    squat_index = self.squat_dropdown.findText(self.preferred_lifts["Squat"])
    vertical_press_index = self.vertical_press_dropdown.findText(self.preferred_lifts["Vertical Press"])

    self.horizontal_press_dropdown.setCurrentIndex(horizontal_press_index)
    self.floor_pull_dropdown.setCurrentIndex(floor_pull_index)
    self.squat_dropdown.setCurrentIndex(squat_index)
    self.vertical_press_dropdown.setCurrentIndex(vertical_press_index)

  def save_preferred_lifts(self):
    horizontal_press = str(self.horizontal_press_dropdown.currentText())
    floor_pull = str(self.floor_pull_dropdown.currentText())
    squat = str(self.squat_dropdown.currentText())
    vertical_press = str(self.vertical_press_dropdown.currentText())
    new_preferred_lifts = {"Horizontal Press": horizontal_press, "Floor Pull":floor_pull,
                           "Squat": squat, "Vertical Press": vertical_press}
    update_preferred_lifts(new_preferred_lifts)
    update_1RM_and_lifts_for_reps()
    self.change_lifts_signal.emit(True)
    self.close()



import math

class OneRepMaxCalculator:
  def __init__(self, weight, repetitions):
    self.weight = int(weight)
    self.repetitions = int(repetitions)
    self.percentages = self.calculate()

  def calculate(self):
    one_rep_max = self.weight*(36/(37-self.repetitions)) # Brzycki formula 
    percentages = [int(one_rep_max*50/100), int(one_rep_max*60/100),
                            int(one_rep_max*70/100), int(one_rep_max*80/100),
                            int(one_rep_max*90/100), int(one_rep_max),
                            int(one_rep_max*102/100)]

    for percentage in percentages:
      index = percentages.index(percentage)
      # round number to next nearest ten if last digit is greater than 6
      if int(str(percentage)[-1]) > 6: percentage = math.ceil(percentage/10.0)*10
      # round number to previous nearest ten if last digit is smaller than 4
      elif int(str(percentage)[-1]) < 4: percentage = math.floor(percentage/10.0)*10
      # if digit is 4, 5 or 6, round last digit to 5
      else: percentage = int(str(percentage)[:-1]+str(5))
      percentages[index] = percentage
    
    return percentages

  def results(self):
    return self.percentages



from PyQt5.QtWidgets import QWidget, QGridLayout, QLabel
from PyQt5.QtGui import QFont
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from .exercises.exercises import Exercises
from .exercises.exercise_info import Exercise
from .header import Header
from fitness_tracker.homepage.side_panel import SidePanel
from .main_panel import MainPanel

class WorkoutsNotes(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.header = Header(self, "Workout Notes")
    self.main_panel = MainPanel(self)
    self.main_panel.show_muscle_group_signal.connect(lambda muscle_group: self.show_muscle_group_layout(muscle_group))
    self.side_panel = SidePanel(self)
    self.side_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name))
    self.create_grid()

  def create_grid(self):
    self.grid = QGridLayout()
    self.grid.addWidget(self.header, 0, 0, 1, 4)
    self.grid.addWidget(self.side_panel, 1, 0, 8, 1)
    self.grid.addWidget(self.main_panel, 1, 1, 8, 3)
    self.setLayout(self.grid)
  
  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)

  @pyqtSlot(str)
  def show_muscle_group_layout(self, muscle_group):
    self.grid.itemAt(2).widget().setParent(None)
    self.exercises = Exercises(self, muscle_group)
    self.exercises.show_exercise_signal.connect(lambda exercise_info: self.show_exercise_info_layout(exercise_info))
    self.grid.addWidget(self.exercises, 1, 1, 8, 3)
   
  @pyqtSlot(object)
  def show_exercise_info_layout(self, exercise_info):
    self.grid.itemAt(2).widget().setParent(None)
    self.exercise_info = Exercise(self, exercise_info)
    self.grid.addWidget(self.exercise_info, 1, 1, 8, 3)



from functools import partial
import json
from PyQt5.QtWidgets import QLabel, QWidget, QPushButton, QVBoxLayout, QHBoxLayout
from PyQt5.QtGui import QFont
from PyQt5.QtCore import pyqtSlot
from .big_lifts_db import fetch_lift_history, delete_history_entry
from fitness_tracker.user_profile.profile_db import fetch_units

class LiftHistory(QWidget):
  def __init__(self):
    super().__init__()
    self.units = "kg" if fetch_units() == "metric" else "lb"
    self.setWindowTitle("Lift History")
    self.layout = QVBoxLayout()
    self.setLayout(self.layout)
    self.create_history(True, True)

  @pyqtSlot(bool)
  def create_history(self, create, init_layout=False):
    lift_history = fetch_lift_history()
    if create and not lift_history == None:
      if not init_layout: 
        self.delete_history()
      lift_history = json.loads(lift_history)
      self.layouts = [None] * len(lift_history)
      self.labels = [None] * len(lift_history)
      self.delete_buttons = [None] * len(lift_history)

      for i in range(len(lift_history)):
        self.layouts[i] = QHBoxLayout()
        self.labels[i] = QLabel(self)
        self.delete_buttons[i] = QPushButton("X", self)
      
      for j, exercise in enumerate(lift_history):
        try:
          self.labels[j].setText(": ".join([exercise[0], " ".join([exercise[1], self.units])]))
        except TypeError: # joining lift for reps as 1RM lift 
          self.labels[j].setText(": ".join([exercise[0], " ".join(["x".join(exercise[1]), self.units])]))
        
        self.delete_buttons[j].setProperty("entry_index", exercise[-1])
        self.delete_buttons[j].clicked.connect(partial(self.delete_history_entry, j, self.delete_buttons[j].property("entry_index")))
        
        self.layouts[j].addWidget(self.labels[j])
        self.layouts[j].addWidget(self.delete_buttons[j])
        self.layout.addLayout(self.layouts[j])

  def delete_history(self):
    index = self.layout.count()
    for i in reversed(range(index)):
      history_entry = self.layout.itemAt(i).layout()
      history_entry_index = history_entry.count()
      for j in reversed(range(history_entry_index)):
        history_entry_widget = history_entry.itemAt(j).widget()
        history_entry_widget.setParent(None)
      self.layout.removeItem(self.layouts[i])
  
  def delete_history_entry(self, i, entry_index):
    self.labels[i].setParent(None)
    self.delete_buttons[i].setParent(None)
    self.layout.removeItem(self.layouts[i])
    delete_history_entry(entry_index)



from PyQt5.QtWidgets import (QWidget, QGridLayout, QLabel, QLineEdit, QComboBox, QPushButton,
                            QFrame, QHBoxLayout, QVBoxLayout)
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt
from .profile_db import *

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.user_data = list(fetch_local_user_data())
    self.create_panel()

  def create_panel(self):
    main_panel_layout = QGridLayout()
    main_panel_layout.addWidget(self.create_top_panel(), 0, 0, 1, 1)
    main_panel_layout.addLayout(self.settings_layout(), 1, 0, 1, 1)
    self.setLayout(main_panel_layout)

  def create_top_panel(self):
    top_layout = QHBoxLayout()
    top_left_layout = QVBoxLayout()
    user_info_label = QLabel("User Info")
    user_info_label.setFont(QFont("Ariel", 16))
    top_left_layout.addWidget(user_info_label)

    name_layout = QHBoxLayout()
    name_label = QLabel("Name:")
    name = QLabel(self.user_data[0])
    name_label.setFont(QFont("Ariel", 10))
    name.setFont(QFont("Ariel", 10))

    name_layout.addWidget(name_label)
    name_layout.addWidget(name)

    gender_layout = QHBoxLayout()
    gender_label = QLabel("Gender:")
    gender = QLabel(self.user_data[1])
    gender_label.setFont(QFont("Ariel", 10))
    gender.setFont(QFont("Ariel", 10))

    gender_layout.addWidget(gender_label)
    gender_layout.addWidget(gender)

    weight_layout = QHBoxLayout()
    weight_label = QLabel("Weight:")
    self.weight = QLabel(set_weight(self.user_data))
    weight_label.setFont(QFont("Ariel", 10))
    self.weight.setFont(QFont("Ariel", 10))
   
    weight_layout.addWidget(weight_label)
    weight_layout.addWidget(self.weight)
    
    height_layout = QHBoxLayout()
    height_label = QLabel("Height:")
    height_label.setFont(QFont("Ariel", 10))
    height_line_edit = QLineEdit()
    height_unit = QComboBox()
    
    height_layout.addWidget(height_label)
    height_layout.addWidget(height_line_edit)
    height_layout.addWidget(height_unit)
    
    age_layout = QHBoxLayout()
    age_label = QLabel("Age:")
    age_label.setFont(QFont("Ariel", 10))
    age_value = QLabel("20")
    edit_age_button = QPushButton("Edit Age")
    
    age_layout.addWidget(age_label)
    age_layout.addWidget(age_value)
    age_layout.addWidget(edit_age_button)

    birthday_layout = QHBoxLayout()
    birthday_label = QLabel("Birthday:")
    birthday_label.setFont(QFont("Ariel", 10))
    birthday_day_combobox = QComboBox()
    birthday_month_combobox = QComboBox()
    
    birthday_layout.addWidget(birthday_label)
    birthday_layout.addWidget(birthday_day_combobox)
    birthday_layout.addWidget(birthday_month_combobox)

    top_left_layout.addLayout(name_layout)
    top_left_layout.addLayout(gender_layout)
    top_left_layout.addLayout(weight_layout)
    top_left_layout.addLayout(height_layout)
    top_left_layout.addLayout(age_layout)
    top_left_layout.addLayout(birthday_layout)

    top_layout.addLayout(top_left_layout)

    framed_top_layout = QFrame()
    framed_top_layout.setFrameStyle(QFrame.StyledPanel)
    framed_top_layout.setLayout(top_layout)
    return framed_top_layout

  def settings_layout(self):
    settings = QVBoxLayout()
    settings_label = QLabel("Settings")
    settings_label.setFont(QFont("Ariel", 14))
    settings.addWidget(settings_label)
    
    settings_layout = QHBoxLayout()
    settings_left_layout = QVBoxLayout()
    settings_right_layout = QVBoxLayout()

    display_units = QHBoxLayout()
    display_units_label = QLabel("Display Units:")
    display_units_label.setFont(QFont("Ariel", 10))
    self.display_units_combobox = QComboBox()
    self.display_units_combobox.setCursor(QCursor(Qt.PointingHandCursor))
    self.display_units_combobox.addItems(["Metric", "Imperial"])
    
    current_index = 0 if fetch_units() == "metric" else 1
    self.display_units_combobox.setCurrentIndex(current_index)
    self.display_units_combobox.activated.connect(lambda: self.change_units(self.display_units_combobox.currentText()))
    
    display_units.addWidget(display_units_label)
    display_units.addWidget(self.display_units_combobox)

    app_style = QHBoxLayout()
    app_style_label = QLabel("Theme")

    settings_left_layout.addLayout(display_units)

    framed_left_layout = QFrame()
    framed_left_layout.setFrameStyle(QFrame.StyledPanel)
    framed_left_layout.setLayout(settings_left_layout)

    settings_layout.addWidget(framed_left_layout)
    settings_layout.addLayout(settings_right_layout)

    settings.addLayout(settings_layout)
    return settings

  def change_units(self, selected_units):
    # don't do anything if current units are metric or imperial
    # and users clicks again on metric or imperial
    if selected_units == "Metric" and self.user_data[2] == "metric":
      pass
    elif selected_units ==  "Imperial" and self.user_data[2] == "imperial":
      pass
    else:
      # update units in user table
      update_units()
      # get updated units
      updated_units = fetch_units()
      current_weight = float(self.user_data[4])
      converted_weight = convert_weight(self.user_data[3], current_weight)
      # update self.user_data
      self.user_data[4] = converted_weight
      self.user_data[3] = updated_units
      # update weight in user table
      update_weight(converted_weight)
      self.weight.setText(set_weight(self.user_data))



from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout
from PyQt5.QtGui import QFont
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from .main_panel import MainPanel
from fitness_tracker.homepage.header import Header
from fitness_tracker.homepage.side_panel import SidePanel

class Profile(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.header = Header(self, "Profile")
    self.main_panel = MainPanel(self)
    self.side_panel = SidePanel(self)
    self.side_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name))
    self.panel_grid()

  def panel_grid(self):
    panel_grid = QGridLayout()
    panel_grid.addWidget(self.header, 0, 0, 1, 4)
    panel_grid.addWidget(self.side_panel, 1, 0, 8, 1)
    panel_grid.addWidget(self.main_panel, 1, 1, 8, 3)
    self.setLayout(panel_grid)

  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)



import os
from functools import partial
from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QPushButton, QLineEdit, QHBoxLayout, QVBoxLayout, QFrame
from PyQt5.QtGui import QIcon, QCursor
from PyQt5.QtCore import QSize, Qt, pyqtSignal
from ..workouts_api import WorkoutsAPI
from .exercise_info import Exercise

path = os.path.join(os.path.abspath(os.path.dirname(__file__)))
image_path = os.path.join(path, "exercise_images") 

class Exercises(QWidget):
  show_exercise_signal = pyqtSignal(object)

  def __init__(self, parent, muscle_group):
    super().__init__(parent)
    self.api = WorkoutsAPI()
    self.muscle_group = muscle_group
    self.exercises = self.api.fetch_exercises(muscle_group, limit=10)
    self.api.download_exercise_images(self.exercises)
    self.create_panel()
    
  def create_panel(self):
    self.grid = QGridLayout()
    
    search_bar_layout = QHBoxLayout()
    search_bar_label = QLabel("Search", self)
    search_bar = QLineEdit(self)
    search_button = QPushButton()
    search_bar_layout.addWidget(search_bar_label)
    search_bar_layout.addWidget(search_bar)
    search_bar_layout.addWidget(search_button)
    self.grid.addLayout(search_bar_layout, 0, 0)
    
    exercise_layouts = [None] * len(self.exercises)
    exercise_images = [None] * len(self.exercises)
    exercise_labels = [None] * len(self.exercises)
     
    i, row = 1, 1
    for j, exercise in enumerate(self.exercises):
      if len(exercise["image"]) == 0: continue
      exercise_layouts[j] = QVBoxLayout()
      exercise_images[j] = QPushButton()
      exercise_labels[j] = QLabel()

      image_name = os.path.basename(exercise["image"][0])
      exercise_image_path = os.path.join(image_path, image_name)
      exercise_images[j].setProperty("exercise_info", exercise)
      exercise_images[j].clicked.connect(partial(self.show_exercise_signal.emit, exercise_images[j].property("exercise_info")))
      exercise_images[j].setIcon(QIcon(exercise_image_path))
      exercise_images[j].setIconSize(QSize(250, 150))
      exercise_images[j].resize(250, 150)
      exercise_images[j].setCursor(Qt.PointingHandCursor)
      exercise_images[j].setFlat(True)
      exercise_labels[j].setText(exercise["name"])
      exercise_labels[j].setAlignment(Qt.AlignCenter)

      exercise_layouts[j].addWidget(exercise_images[j])
      exercise_layouts[j].addWidget(exercise_labels[j])
      i += 1
      self.grid.addLayout(exercise_layouts[j], row, i)
      if i == 3:
        i = 0
        row += 1
    self.setLayout(self.grid)



from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout, QVBoxLayout, QLabel, QCalendarWidget, QFrame, QComboBox
from PyQt5.QtCore import Qt

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.create_panel()

  def create_panel(self):
    grid = QGridLayout()
    
    calendar_frame = QFrame()
    calendar_frame.setFixedHeight(210)
    calendar_frame.setFrameStyle(QFrame.StyledPanel)
    
    calendar_layout = QHBoxLayout()
    calendar_layout.addLayout(self.create_description())
    calendar_layout.addWidget(self.create_calendar())
    calendar_frame.setLayout(calendar_layout)
    
    grid.addWidget(calendar_frame, 0, 0, 1, 1)
    grid.addWidget(self.create_graphs(), 1, 0, 4, 2)
    self.setLayout(grid)

  def create_description(self):
    notes_description = QVBoxLayout()
    notes_label = QLabel("Store your progress to desired weight. \nWeight loss notes offer you simple way of \ntracking your weight progress. \nYou can see your progression visualized on graphs.")
    notes_label.setFixedHeight(100)
    notes_description.addWidget(notes_label)
    return notes_description

  def create_calendar(self):
    calendar_frame = QFrame()
    calendar_layout = QVBoxLayout()
    calendar = QCalendarWidget(self)
    calendar_layout.addWidget(calendar)
    calendar_frame.setLayout(calendar_layout)
    return calendar_frame

  def create_graphs(self):
    graph_layout = QHBoxLayout()

    graph1_layout = QVBoxLayout()
    graph1 = QWidget()
    graph1.setStyleSheet("background-color: gold")
    dropdown1 = QComboBox()
    dropdown1.addItems(["Daily", "Weekly", "Monthly", "Yearly"])
    graph1_layout.addWidget(graph1)
    graph1_layout.addWidget(dropdown1)
    
    graph2_layout = QVBoxLayout()
    graph2 = QWidget()
    graph2.setStyleSheet("background-color: orange")
    dropdown2 = QComboBox()
    dropdown2.addItems(["Daily", "Weekly", "Monthly", "Yearly"])
    graph2_layout.addWidget(graph2)
    graph2_layout.addWidget(dropdown2)

    graph_layout.addLayout(graph1_layout)
    graph_layout.addLayout(graph2_layout)
    frame = QFrame()
    frame.setFrameStyle(QFrame.StyledPanel)
    frame.setLayout(graph_layout)
    return frame



import re
import sqlite3
import string
import hashlib
import psycopg2
from psycopg2 import sql
from fitness_tracker.config import db_path, db_info

def check_valid_email(email):
  # basic validation, checks for example@gmail.com and similar format
  # for real validation, email confirmation is needed
  email_regex = re.compile(r"^[\w\.\+\-]+\@[\w]+\.[a-z]{2,6}$")
  return not re.match(email_regex, email) == None
 
def check_valid_password(password):
  valid = True
  valid_characters = set(string.ascii_letters + string.digits + '@#$%^&+=')
  if (len(password) < 8):
    valid = False
    print("Password is too short.")
  elif any(char not in valid_characters for char in password):
    valid = False
    print("Password contains invalid characters.")
  return valid

def create_user(user_email, user_password):
  status = True
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    try:
      with conn.cursor() as cursor:
        user_password = hashlib.sha256(user_password.encode('utf-8')).hexdigest()
        cursor.execute("INSERT INTO users (email, password) VALUES (%s, %s)", (user_email, user_password,))
    except psycopg2.errors.UniqueViolation: # user with given email already exists
      status = False
  return status

def create_user_info_after_signup(user_info, email, db_path=db_path):
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      update_query = create_update_query("users", email, user_info)
      cursor.execute(update_query)
    
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      update_query = create_update_query("users",  email, user_info, sqlite=True)
      cursor.execute(update_query)

def create_update_query(table_name, email, columns_values_dict, sqlite=False):
  update_query = "UPDATE {table} SET"
  # create list with [column_name, value] pairs
  pairs = [[pair[0], pair[1]] for pair in zip(columns_values_dict.keys(), columns_values_dict.values())]
  for pair in pairs:
    pair[1] = "'%s'" % pair[1] # add ticks to value
    joined_pair = "=".join([pair[0], pair[1]]) # create column='value' string
    # if its first pair, join it to update_query with blank space
    if pairs.index(pair) == 0: update_query = " ".join([update_query, joined_pair])
    # if its last pair, add WHERE email={email} to the end of update_query
    elif pairs.index(pair) == len(pairs)-1:
      update_query = ", ".join([update_query, joined_pair])
      update_query = " ".join([update_query, "WHERE email='%s'"])
    else: update_query = ", ".join([update_query, joined_pair])
  if sqlite: # make adjustments for sqlite3
    update_query = update_query.replace("{table}", "'%s'")
    return update_query % (table_name, email)
  else: # make sql query for psycopg2
    update_query = update_query.format(table=table_name)
    return update_query % email

def create_user_table(email, password, db_path=db_path):
  status = True
  password = hashlib.sha256(password.encode('UTF-8')).hexdigest()
  # set user table columns here
  create_table = """
                 CREATE TABLE 'users' (
                 email text NOT NULL,
                 password text NOT NULL,
                 name text,
                 age text,
                 gender text,
                 units text,
                 weight text,
                 height text,
                 goal text,
                 goalparams text,
                 goalweight text,
                 logged_in text,
                 ID integer NOT NULL,
                 PRIMARY KEY (ID));
                 """ 
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute(create_table)
    cursor.execute("INSERT INTO 'users' (email, password) VALUES (?, ?)", (email, password,))
    cursor.execute("UPDATE 'users' SET logged_in='YES' WHERE email=?", (email,))



import sqlite3
import psycopg2
from psycopg2 import sql
from fitness_tracker.user_profile.profile_db import logged_in_user_email
from fitness_tracker.config import db_path, db_info

def table_is_empty(db_path=db_path):
  email = logged_in_user_email(db_path) 
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT COUNT (*) FROM 'nutrition' WHERE email=?", (email,))
    if cursor.fetchone()[0] == 0: return True
  return False

def table_exists(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='nutrition'")
    if cursor.fetchone()[0] == 0: return False
  return True

def fetch_calorie_goal():
  email = logged_in_user_email(db_path) 
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT calorie_goal FROM 'nutrition' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_nutrition_data(db_path=db_path):
  email = logged_in_user_email(db_path) 
  calorie_goal = None
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("SELECT calorie_goal FROM nutrition WHERE email=%s", (email,))
      calorie_goal = cursor.fetchone()[0]
  if table_is_empty(db_path):
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute("INSERT INTO 'nutrition' (email, calorie_goal) VALUES (?, ?)", (email, calorie_goal,))

def update_calorie_goal(calorie_goal, db_path=db_path):
  email = logged_in_user_email(db_path) 
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'nutrition' SET calorie_goal=?", (calorie_goal,))
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE nutrition SET calorie_goal=%s WHERE email=%s", (calorie_goal, email,))

def insert_calorie_goal(calorie_goal):
  email = logged_in_user_email(db_path) 
  insert = "INSERT INTO nutrition ({columns}) VALUES %s"
  columns = sql.SQL(", ").join(sql.Identifier(column) for column in ["email", "calorie_goal"])
  values = (email, calorie_goal)
  try:
    with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                          user=db_info["user"], password=db_info["password"]) as conn:
      with conn.cursor() as cursor:
        cursor.execute(sql.SQL(insert).format(columns=columns), (values,))
    
    insert = "INSERT INTO 'nutrition' {columns} VALUES {values}"
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute(insert.format(columns=("email", "calorie_goal"), values=values))
  except psycopg2.errors.UniqueViolation:
    fetch_nutrition_data(db_path)

def create_nutrition_table(db_path=db_path):
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    create_table = """
                   CREATE TABLE IF NOT EXISTS
                   'nutrition' (
                   email text,
                   calorie_goal text,
                   ID integer NOT NULL,
                   PRIMARY KEY(ID));
                   """
    cursor.execute(create_table)



from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout, QVBoxLayout, QLabel, QGroupBox, QRadioButton
from PyQt5.QtGui import QFont

class Header(QWidget):
  def __init__(self, parent, title):
    super().__init__(parent)
    self.title = title
    self.create_header()
  
  def create_header(self):
    grid = QGridLayout()
    self.setLayout(grid)
    
    title_layout = QHBoxLayout()
    title_label = QLabel(self.title, self)
    title_label.setFont(QFont("Arial", 25))
    title_layout.addWidget(title_label)

    qw_layout = QHBoxLayout()
    qw_group = QGroupBox("Quick View:")
    radio_layout = QHBoxLayout()
    
    s_radio = QRadioButton("Strength")
    s_radio.setChecked(True)
    wp_radio = QRadioButton("Weight progression")
    n_radio = QRadioButton("Nutrition")

    radio_layout.addWidget(s_radio)
    radio_layout.addWidget(wp_radio)
    radio_layout.addWidget(n_radio)
    
    qw_group.setLayout(radio_layout)
    qw_layout.addWidget(qw_group)

    u_layout = QHBoxLayout()
    units_layout = QVBoxLayout()
    units_group = QGroupBox("Units:")

    m_radio = QRadioButton("Metric")
    m_radio.setChecked(True)
    i_radio = QRadioButton("Imperial")

    units_layout.addWidget(m_radio)
    units_layout.addWidget(i_radio)

    units_group.setLayout(units_layout)
    u_layout.addWidget(units_group)

    grid.addLayout(title_layout, 0, 0, 1, 1)
    grid.addLayout(qw_layout, 0, 3, 1, 3)
    grid.addLayout(u_layout, 0, 7, 1, 1)



import os
import json
from functools import partial
from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout, QVBoxLayout, QLabel, QFrame, QCalendarWidget, QPushButton, QMessageBox
from PyQt5.QtGui import QFont, QIcon, QCursor
from PyQt5.QtCore import Qt, QLocale, QSize, pyqtSignal, pyqtSlot
from .exercises.exercises import Exercises
from .create_workout_window import CreateWorkoutWindow1, CreateWorkoutWindow2, EditWorkout
from .workouts_db import (create_workouts_table, insert_default_workouts_data,
                          table_is_empty, fetch_current_workout_plan, fetch_workouts,
                          delete_workout, update_current_workout)

path = os.path.join(os.path.abspath(os.path.dirname(__file__)))
muscle_groups_path = os.path.join(path, "muscle_groups")
icons_path = os.path.join(path, "icons")

muscle_groups = {"Chest": os.path.join(muscle_groups_path, "chest.svg"),
                 "Back": os.path.join(muscle_groups_path, "back.svg"),
                 "Core": os.path.join(muscle_groups_path, "core.svg"),
                 "Biceps": os.path.join(muscle_groups_path, "biceps.svg"),
                 "Triceps": os.path.join(muscle_groups_path, "triceps.svg"),
                 "Quadriceps": os.path.join(muscle_groups_path, "quads.svg"),
                 "Calves": os.path.join(muscle_groups_path, "calves.svg"),
                 "Shoulders": os.path.join(muscle_groups_path, "shoulders.svg"),
                 "Gluteus": os.path.join(muscle_groups_path, "gluteus.svg"),
                 "Hamstrings": os.path.join(muscle_groups_path, "hamstrings.svg")}

class MainPanel(QWidget):
  show_muscle_group_signal = pyqtSignal(str)

  def __init__(self, parent):
    super().__init__()
    create_workouts_table()
    if table_is_empty(): insert_default_workouts_data()
    self.current_workout_plan = fetch_current_workout_plan()
    if self.current_workout_plan == "":
      self.current_workout_plan = "None"
    self.fetched_workouts = json.loads(fetch_workouts())
    # {WorkoutName: {("CreateWorkoutWindow2", CreateWorkoutWindow2): {Day: EditWorkout object}}}
    self.workouts = self.generate_workouts_objects()
    self.create_panel()
      
  def create_panel(self):
    self.grid = QGridLayout()
    self.grid.addWidget(self.create_stats(), 0, 0, 1, 2)
    self.grid.addWidget(self.create_calendar_workouts(), 1, 0, 1, 2)
    self.grid.addWidget(self.create_exercises(), 2, 0, 1, 2)
    self.setLayout(self.grid)
    
  def generate_workouts_objects(self):
    workouts = {}
    if len(self.fetched_workouts) > 0:
      for workout in self.fetched_workouts:
        workout_name = workout
        workout_days = list(self.fetched_workouts[workout].keys())
        create_workout_window_2 = CreateWorkoutWindow2(workout_name, workout_days, empty_workout=False)
        create_workout_window_2.show_existing_workout_edit.connect(lambda s: self.show_existing_workout_edit(s))
        days = {}
        for workout_day in self.fetched_workouts[workout].keys():
           workout_day_info = self.fetched_workouts[workout][workout_day]
           edit_workout_object = EditWorkout(workout_day, workout_day_info)
           edit_workout_object.update_workout_day_signal.connect(lambda s: create_workout_window_2.add_workouts(s))
           days[workout_day] = edit_workout_object
        workouts[workout] = {("CreateWorkoutWindow2", create_workout_window_2): days}
    
    i = 0
    while len(workouts) < 3:
      workouts[str(i)] = "empty_workout"
      i += 1
    return workouts
  
  @pyqtSlot(object)
  def show_create_window_2(self, signal):
    self.create_workout_window1.close()
    workout_name = signal[0]
    workout_days = signal[1]
    current_workout = signal[2]
    self.create_workout_window2 = CreateWorkoutWindow2(workout_name, workout_days, set_as_current_workout=current_workout, empty_workout=True)
    self.create_workout_window2.show_edit_workout.connect(lambda s: self.show_edit_workout_window(s))
    self.create_workout_window2.refresh_layout_signal.connect(lambda s: self.refresh_layout(s)) 
    
    self.edit_windows = {}
    for day in signal[1]:
      self.edit_windows[day] = EditWorkout(day)
      self.edit_windows[day].update_workout_day_signal.connect(lambda s: self.create_workout_window2.add_workouts(s))
    
    self.create_workout_window2.show()
  
  @pyqtSlot(str)
  def show_edit_workout_window(self, workout_day):
    self.edit_windows[workout_day].show()  
  
  def show_empty_workout_edit(self):
    self.create_workout_window1 = CreateWorkoutWindow1()
    self.create_workout_window1.continue_signal.connect(lambda s: self.show_create_window_2(s))
    self.create_workout_window1.show()

  def show_existing_workout(self, workout):
    create_workout_window_2 = list(self.workouts[workout].keys())[0][1]
    create_workout_window_2.show()
  
  @pyqtSlot(object)
  def show_existing_workout_edit(self, workout):
    workout_name = workout[0]
    workout_day = workout[1]
    for key in self.workouts[workout_name]:
      for day in self.workouts[workout_name][key]:
        if day == workout_day:
          self.workouts[workout_name][key][day].show()

  @pyqtSlot(bool)
  def refresh_layout(self, signal):
    if signal:
      self.fetched_workouts = json.loads(fetch_workouts())
      self.workouts = self.generate_workouts_objects()
      self.current_workout_plan = fetch_current_workout_plan()
      
      if self.current_workout_plan == "" or self.fetched_workouts == {}:
        self.current_workout_plan = ""
      elif not self.current_workout_plan in self.fetched_workouts:
        next_workout = list(self.workouts.keys())[0]
        self.current_workout_plan = next_workout
      update_current_workout(self.current_workout_plan, True)
      self.current_workout_label2.setText(self.current_workout_plan)
       
      old_workouts_reference = self.grid.itemAt(1).widget()
      new_calendar_workouts_frame = self.create_calendar_workouts()
      self.grid.replaceWidget(old_workouts_reference, new_calendar_workouts_frame)
      old_workouts_reference.setParent(None)
  
  def show_delete_workout_dialog(self, workout):
    message_box = QMessageBox()
    message_box.setIcon(QMessageBox.Question)
    message_box.setText("Are you sure you want to delete this workout?")
    message_box.setWindowTitle("Confirm delete")
    message_box.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
    message_box.buttonClicked.connect(lambda answer: self.delete_existing_workout(answer.text(), workout))
    message_box.exec_()
  
  def delete_existing_workout(self, answer, workout):
    if "OK" in answer:
      delete_workout(workout)
      self.refresh_layout(True)

  def create_stats(self):
    frame = QFrame()
    frame.setProperty("name", "Stats frame 1")
    frame.setFrameStyle(QFrame.StyledPanel)

    current_workout_frame = QFrame()
    current_workout_frame.setFrameStyle(QFrame.StyledPanel)
    current_workout_layout = QVBoxLayout()

    current_workout_label = QLabel("Current workout plan", self)
    current_workout_label.setAlignment(Qt.AlignCenter)
    self.current_workout_label2 = QLabel(self.current_workout_plan, self)
    self.current_workout_label2.setAlignment(Qt.AlignCenter)

    current_workout_layout.addWidget(current_workout_label)
    current_workout_layout.addWidget(self.current_workout_label2)

    current_workout_frame.setLayout(current_workout_layout)

    last_workout_frame = QFrame()
    last_workout_frame.setFrameStyle(QFrame.StyledPanel)
    last_workout_layout = QVBoxLayout()

    last_workout_label = QLabel("Last workout", self)
    last_workout_label.setAlignment(Qt.AlignCenter)
    last_workout_label2 = QLabel("Monday, 9.2.2017", self)
    last_workout_label2.setAlignment(Qt.AlignCenter)

    last_workout_layout.addWidget(last_workout_label)
    last_workout_layout.addWidget(last_workout_label2)

    last_workout_frame.setLayout(last_workout_layout)

    number_of_workouts_frame = QFrame()
    number_of_workouts_frame.setFrameStyle(QFrame.StyledPanel)
    number_of_workouts_layout = QVBoxLayout()

    number_of_workouts_label = QLabel("Total number of workouts", self)
    number_of_workouts_label.setAlignment(Qt.AlignCenter)
    number_of_workouts_label2 = QLabel("31", self)
    number_of_workouts_label2.setAlignment(Qt.AlignCenter)

    number_of_workouts_layout.addWidget(number_of_workouts_label)
    number_of_workouts_layout.addWidget(number_of_workouts_label2)

    number_of_workouts_frame.setLayout(number_of_workouts_layout)

    wrapper = QHBoxLayout()
    wrapper.addWidget(current_workout_frame)
    wrapper.addWidget(last_workout_frame)
    wrapper.addWidget(number_of_workouts_frame)
    frame.setLayout(wrapper)
    return frame

  def create_calendar_workouts(self):
    frame = QFrame()
    frame.setProperty("name", "Calendar frame 2")
    frame.setFrameStyle(QFrame.StyledPanel)
    layout = QHBoxLayout()
    
    calendar = QCalendarWidget(self)
    calendar.setLocale(QLocale(QLocale.English))
    calendar.setFirstDayOfWeek(Qt.DayOfWeek.Monday)

    layout.addWidget(calendar)
    layout.addWidget(self.create_my_workouts())
    frame.setLayout(layout)
    return frame   
  
  def create_my_workouts(self):
    my_workouts_frame = QFrame()
    my_workouts_frame.setFrameStyle(QFrame.StyledPanel)
    my_workouts_layout = QVBoxLayout()
    my_workouts_label = QLabel("My Workouts", self)
    my_workouts_layout.addWidget(my_workouts_label)

    workout_buttons_layouts = [None] * 3
    workout_buttons = [None] * 3
    workout_buttons_frames = [None] * 3
    delete_buttons = [None] * 3
    
    i = 0 
    for workout in self.workouts:
      workout_buttons_frames[i] = QFrame()   
      workout_buttons_frames[i].setFrameStyle(QFrame.StyledPanel)
      workout_buttons_layouts[i] = QHBoxLayout()
      
      delete_buttons[i] = QPushButton(QIcon(os.path.join(icons_path, "x.png")), "", self)
      delete_buttons[i].setStyleSheet("border: none")
      delete_buttons[i].setCursor(QCursor(Qt.PointingHandCursor))
      
      if self.workouts[workout] == "empty_workout":
        workout_buttons[i] = QPushButton("<empty workout> 0 days", self)
        workout_buttons[i].clicked.connect(lambda: self.show_empty_workout_edit())
      else:
        key_for_days = list(self.workouts[workout].keys())[0]
        days = len(self.workouts[workout][key_for_days])
        if days == 1: string_days = "day"
        else: string_days = "days"
        workout_buttons[i] = QPushButton(" ".join([workout, str(days), string_days]))
        workout_buttons[i].clicked.connect(partial(self.show_existing_workout, workout))
        delete_buttons[i].clicked.connect(partial(self.show_delete_workout_dialog, workout))

      workout_buttons[i].setFlat(True)
      workout_buttons[i].setStyleSheet("color: white")
      workout_buttons[i].setCursor(QCursor(Qt.PointingHandCursor))
      
      workout_buttons_layouts[i].addWidget(workout_buttons[i])
      workout_buttons_layouts[i].addWidget(delete_buttons[i])
      workout_buttons_frames[i].setLayout(workout_buttons_layouts[i])
      my_workouts_layout.addWidget(workout_buttons_frames[i]) 
    
    my_workouts_frame.setLayout(my_workouts_layout)
    return my_workouts_frame

  def create_exercises(self):
    frame = QFrame()
    frame.setProperty("name", "Exercises frame 3")
    frame.setFrameStyle(QFrame.StyledPanel)
    exercises_layout = QVBoxLayout()

    label = QLabel("Exercises", self)
    label.setFont(QFont("Ariel", 15))
    
    first_row = QHBoxLayout()

    chest_muscle_group = QVBoxLayout()
    chest_image = QPushButton(self)
    chest_image.setIcon(QIcon(muscle_groups["Chest"]))
    chest_image.setIconSize(QSize(100, 60))
    chest_image.resize(100, 60)
    chest_image.setStyleSheet("border: none")
    chest_image.setCursor(Qt.PointingHandCursor)
    chest_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Chest"))

    chest_label = QLabel("Chest", self)
    chest_label.setAlignment(Qt.AlignCenter)
    chest_muscle_group.addWidget(chest_image)
    chest_muscle_group.addWidget(chest_label)
    
    back_muscle_group = QVBoxLayout()
    back_image = QPushButton(self)
    back_image.setIcon(QIcon(muscle_groups["Back"]))
    back_image.setIconSize(QSize(100, 60))
    back_image.resize(100, 60)
    back_image.setStyleSheet("border: none")
    back_image.setCursor(Qt.PointingHandCursor)
    back_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Back"))
    
    back_label = QLabel("Back", self)
    back_label.setAlignment(Qt.AlignCenter)
    back_muscle_group.addWidget(back_image)
    back_muscle_group.addWidget(back_label)
    
    triceps_muscle_group = QVBoxLayout()
    triceps_image = QPushButton(self)
    triceps_image.setIcon(QIcon(muscle_groups["Triceps"]))
    triceps_image.setIconSize(QSize(100, 60))
    triceps_image.resize(100, 60)
    triceps_image.setStyleSheet("border: none")
    triceps_image.setCursor(Qt.PointingHandCursor)
    triceps_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Triceps"))

    triceps_label = QLabel("Triceps", self)
    triceps_label.setAlignment(Qt.AlignCenter)
    triceps_muscle_group.addWidget(triceps_image)
    triceps_muscle_group.addWidget(triceps_label)

    biceps_muscle_group = QVBoxLayout()
    biceps_image = QPushButton(self)
    biceps_image.setIcon(QIcon(muscle_groups["Biceps"]))
    biceps_image.setIconSize(QSize(100, 60))
    biceps_image.resize(100, 60)
    biceps_image.setStyleSheet("border: none")
    biceps_image.setCursor(Qt.PointingHandCursor)
    biceps_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Biceps"))
    
    biceps_label = QLabel("Biceps", self)
    biceps_label.setAlignment(Qt.AlignCenter)
    biceps_muscle_group.addWidget(biceps_image)
    biceps_muscle_group.addWidget(biceps_label)
    
    shoulders_muscle_group = QVBoxLayout()
    shoulders_image = QPushButton(self)
    shoulders_image.setIcon(QIcon(muscle_groups["Shoulders"]))
    shoulders_image.setIconSize(QSize(100, 60))
    shoulders_image.resize(100, 60)
    shoulders_image.setStyleSheet("border: none")
    shoulders_image.setCursor(Qt.PointingHandCursor)
    shoulders_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Shoulders"))
    
    shoulders_label = QLabel("Shoulders", self)
    shoulders_label.setAlignment(Qt.AlignCenter)
    shoulders_muscle_group.addWidget(shoulders_image)
    shoulders_muscle_group.addWidget(shoulders_label)

    first_row.addLayout(chest_muscle_group)
    first_row.addLayout(back_muscle_group)
    first_row.addLayout(triceps_muscle_group)
    first_row.addLayout(biceps_muscle_group)
    first_row.addLayout(shoulders_muscle_group)
    
    second_row = QHBoxLayout()
    
    core_muscle_group = QVBoxLayout()
    core_image = QPushButton(self)
    core_image.setIcon(QIcon(muscle_groups["Core"]))
    core_image.setIconSize(QSize(100, 60))
    core_image.resize(100, 60)
    core_image.setStyleSheet("border: none")
    core_image.setCursor(Qt.PointingHandCursor)
    core_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Core"))
    
    core_label = QLabel("Core", self)
    core_label.setAlignment(Qt.AlignCenter)
    core_muscle_group.addWidget(core_image)
    core_muscle_group.addWidget(core_label)
    
    gluteus_muscle_group = QVBoxLayout()
    gluteus_image = QPushButton(self)
    gluteus_image.setIcon(QIcon(muscle_groups["Gluteus"]))
    gluteus_image.setIconSize(QSize(100, 60))
    gluteus_image.resize(100, 60)
    gluteus_image.setStyleSheet("border: none")
    gluteus_image.setCursor(Qt.PointingHandCursor)
    gluteus_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Legs"))
    
    gluteus_label = QLabel("Gluteus", self)
    gluteus_label.setAlignment(Qt.AlignCenter)
    gluteus_muscle_group.addWidget(gluteus_image)
    gluteus_muscle_group.addWidget(gluteus_label)

    quadriceps_muscle_group = QVBoxLayout()
    quadriceps_image = QPushButton(self)
    quadriceps_image.setIcon(QIcon(muscle_groups["Quadriceps"]))
    quadriceps_image.setIconSize(QSize(100, 60))
    quadriceps_image.resize(100, 60)
    quadriceps_image.setStyleSheet("border: none")
    quadriceps_image.setCursor(Qt.PointingHandCursor)
    quadriceps_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Legs"))
    
    quadriceps_label = QLabel("Quadriceps", self)
    quadriceps_label.setAlignment(Qt.AlignCenter)
    quadriceps_muscle_group.addWidget(quadriceps_image)
    quadriceps_muscle_group.addWidget(quadriceps_label)

    calves_muscle_group = QVBoxLayout()
    calves_image = QPushButton(self)
    calves_image.setIcon(QIcon(muscle_groups["Calves"]))
    calves_image.setIconSize(QSize(100, 60))
    calves_image.resize(100, 60)
    calves_image.setStyleSheet("border: none")
    calves_image.setCursor(Qt.PointingHandCursor)
    calves_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Calves"))
    
    calves_label = QLabel("Calves", self)
    calves_label.setAlignment(Qt.AlignCenter)
    calves_muscle_group.addWidget(calves_image)
    calves_muscle_group.addWidget(calves_label)
    
    hamstrings_group = QVBoxLayout()
    hamstrings_image = QPushButton(self)
    hamstrings_image.setIcon(QIcon(muscle_groups["Hamstrings"]))
    hamstrings_image.setIconSize(QSize(100, 60))
    hamstrings_image.resize(100, 60)
    hamstrings_image.setStyleSheet("border: none")
    hamstrings_image.setCursor(Qt.PointingHandCursor)
    hamstrings_image.clicked.connect(lambda: self.show_muscle_group_signal.emit("Legs"))
    
    hamstrings_label = QLabel("Hamstrings", self)
    hamstrings_label.setAlignment(Qt.AlignCenter)
    hamstrings_group.addWidget(hamstrings_image)
    hamstrings_group.addWidget(hamstrings_label)
    
    second_row.addLayout(core_muscle_group)
    second_row.addLayout(gluteus_muscle_group)
    second_row.addLayout(quadriceps_muscle_group)
    second_row.addLayout(calves_muscle_group)
    second_row.addLayout(hamstrings_group)

    exercises_layout.addWidget(label)
    exercises_layout.addLayout(first_row)
    exercises_layout.addLayout(second_row)
    frame.setLayout(exercises_layout)
    return frame



from PyQt5.QtWidgets import QWidget, QGridLayout, QVBoxLayout, QHBoxLayout, QPushButton, QLabel
from PyQt5.QtGui import QFont, QCursor
from PyQt5.QtCore import Qt, pyqtSignal

class Header(QWidget):
  change_layout_signal = pyqtSignal(str)
  
  def __init__(self, parent):
    super().__init__(parent)
    self.setStyleSheet("QPushButton{color:white;background-color: #191716;}")
    self.create_header()
  
  def create_header(self):
    grid = QGridLayout()

    nutrition_title_layout = QVBoxLayout()
    nutrition_label = QLabel("Nutrition")
    nutrition_label.setFont(QFont("Ariel", 15))
    nutrition_label.setStyleSheet("color:white;font-weight:bold;")
    nutrition_label.setFixedHeight(20)

    nutrition_description = QLabel("Lorem ipsum dolor sit amet, consectetur adipiscing elit.")
    nutrition_description.setFont(QFont("Ariel", 9))
    nutrition_description.setStyleSheet("color:grey;")

    nutrition_title_layout.addWidget(nutrition_label)
    nutrition_title_layout.addWidget(nutrition_description)
    
    buttons_layout = QHBoxLayout()
    self.notes_button = QPushButton("Notes")
    self.notes_button.setStyleSheet("background-color: #603A40;")
    self.notes_button.setFixedSize(60, 35)
    self.notes_button.setCursor(QCursor(Qt.PointingHandCursor))
    self.notes_button.clicked.connect(lambda: self.change_layout_signal.emit(self.notes_button.text()))
    self.food_database_button = QPushButton("Food Database")
    self.food_database_button.setFixedSize(110, 35)
    self.food_database_button.setCursor(QCursor(Qt.PointingHandCursor))
    self.food_database_button.clicked.connect(lambda: self.change_layout_signal.emit(self.food_database_button.text()))
    
    buttons_layout.addWidget(self.notes_button)
    buttons_layout.addWidget(self.food_database_button)
    
    grid.addLayout(nutrition_title_layout, 0, 0)
    grid.addLayout(buttons_layout, 0, 1)
    self.setLayout(grid)

  def set_current_layout_button(self, layout):
    if layout == "Food Database":
      self.food_database_button.setStyleSheet("background-color: #603A40;")
      self.notes_button.setStyleSheet("background-color: #191716;")
    elif layout == "Notes":
      self.notes_button.setStyleSheet("background-color: #603A40;")
      self.food_database_button.setStyleSheet("background-color: #191716;")



import sys
import sqlite3
import psycopg2
import json
from psycopg2 import sql
from fitness_tracker.user_profile.profile_db import fetch_units, logged_in_user_email
from fitness_tracker.common.units_conversion import kg_to_pounds, pounds_to_kg
from fitness_tracker.config import db_path, db_info

def table_is_empty(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT COUNT (*) FROM 'big_lifts' WHERE email=?", (email,))
    if cursor.fetchone()[0] == 0: return True
  return False

def fetch_units_from_big_lifts(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT units FROM 'big_lifts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_one_rep_maxes(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("""SELECT one_rep_maxes FROM 'big_lifts' WHERE email=?""", (email,))
    return cursor.fetchone()[0]

def fetch_lifts_for_reps(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT lifts_for_reps FROM 'big_lifts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_preferred_lifts(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT preferred_lifts FROM 'big_lifts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_lift_history(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT lift_history FROM 'big_lifts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_user_big_lifts_table_data(db_path=db_path):
  email = logged_in_user_email(db_path)
  select_1RM = "SELECT one_rep_maxes FROM big_lifts WHERE email=%s"
  select_lifts_for_reps = "SELECT lifts_for_reps FROM big_lifts WHERE email=%s"
  select_preferred_lifts = "SELECT preferred_lifts FROM big_lifts WHERE email=%s"
  select_lift_history = "SELECT lift_history FROM big_lifts WHERE email=%s"
  select_units = "SELECT units FROM big_lifts WHERE email=%s"

  one_rep_maxes = None
  lifts_for_reps = None
  preferred_lifts = None
  lift_history = None
  units = None

  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute(select_1RM, (email,))
      one_rep_maxes = cursor.fetchone()[0]
      
      cursor.execute(select_lifts_for_reps, (email,))
      lifts_for_reps = cursor.fetchone()[0]
      
      cursor.execute(select_preferred_lifts, (email,))
      preferred_lifts = cursor.fetchone()[0]
      
      cursor.execute(select_lift_history, (email,))
      lift_history = cursor.fetchone()[0]

      cursor.execute(select_units, (email,))
      units = cursor.fetchone()[0]
  
  if table_is_empty():
    insert_values = """
                    INSERT INTO 'big_lifts' (email, one_rep_maxes, lifts_for_reps, preferred_lifts, lift_history, units) VALUES
                    (?, ?, ?, ?, ?, ?)
                    """
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute(insert_values, (email, one_rep_maxes, lifts_for_reps, preferred_lifts, lift_history, units,))

def insert_default_values(db_path=db_path):
  default_exercises = ["Bench Press", "Deadlift", "Back Squat", "Overhead Press"]
  email = logged_in_user_email(db_path)
  units = fetch_units(db_path)
  one_RM_dict = json.dumps({exercise:"0" for exercise in default_exercises})
  lifts_for_reps = json.dumps({exercise:["0", "0"] for exercise in default_exercises})
  preferred_lifts = json.dumps({"Horizontal Press": default_exercises[0],
                                               "Floor Pull": default_exercises[1],
                                               "Squat": default_exercises[2],
                                               "Vertical Press": default_exercises[3]})

  default_dict = {"one_rep_maxes": one_RM_dict, "lifts_for_reps": lifts_for_reps,
                  "preferred_lifts": preferred_lifts, "email": email, "units": units}

  try:
    with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                          user=db_info["user"], password=db_info["password"]) as conn:
      with conn.cursor() as cursor:
        insert_query = "INSERT INTO big_lifts ({columns}) VALUES %s"
        columns = sql.SQL(", ").join(sql.Identifier(column) for column in tuple(default_dict.keys()))
        values = tuple(value for value in default_dict.values())
        cursor.execute(sql.SQL(insert_query).format(columns=columns), (values,))
  
      with sqlite3.connect(db_path) as conn:
        cursor = conn.cursor()
        cursor.execute("INSERT INTO 'big_lifts' {columns} VALUES {values}".format(columns=tuple(default_dict.keys()), values=tuple(default_dict.values())))
  except psycopg2.errors.UniqueViolation: # user already has big_lifts table with data on server
    fetch_user_big_lifts_table_data(db_path)

def create_big_lifts_table(db_path=db_path):
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    create_table = """
                   CREATE TABLE IF NOT EXISTS
                   'big_lifts' (
                   email text,
                   one_rep_maxes text,
                   lifts_for_reps text,
                   preferred_lifts text,
                   lift_history text,
                   units text,
                   id integer NOT NULL,
                   PRIMARY KEY(id));
                   """
    cursor.execute(create_table)

def update_preferred_lifts(new_preferred_lifts, db_path=db_path):
  email = logged_in_user_email(db_path)
  new_preferred_lifts = json.dumps(new_preferred_lifts)
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET preferred_lifts=%s WHERE email=%s", (new_preferred_lifts, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET preferred_lifts=? WHERE email=?", (new_preferred_lifts, email,))

# updates exercises
def update_1RM_and_lifts_for_reps(db_path=db_path):
  email = logged_in_user_email(db_path)
  preferred_lifts = list(json.loads(fetch_preferred_lifts(db_path)).values())
  one_rep_maxes = json.loads(fetch_one_rep_maxes(db_path))
  lifts_for_reps = json.loads(fetch_lifts_for_reps(db_path))

  new_one_rep_maxes = json.dumps({preferred_lifts[i]:value for i, value in enumerate(one_rep_maxes.values())})
  new_lifts_for_reps = json.dumps({preferred_lifts[i]:value for i, value in enumerate(lifts_for_reps.values())})
      
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET one_rep_maxes=%s WHERE email=%s", (new_one_rep_maxes, email,))
      cursor.execute("UPDATE big_lifts SET lifts_for_reps=%s WHERE email=%s", (new_lifts_for_reps, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET one_rep_maxes=? WHERE email=?", (new_one_rep_maxes, email,))
    cursor.execute("UPDATE 'big_lifts' SET lifts_for_reps=? WHERE email=?", (new_lifts_for_reps, email,))

# updates weight
def update_1RM_lifts(new_values, db_path=db_path):
  new_values = list(new_values.values()) 
  email = logged_in_user_email(db_path)
  one_rep_max_lifts = json.loads(fetch_one_rep_maxes(db_path))
  for i, lift in enumerate(one_rep_max_lifts.keys()):
    one_rep_max_lifts[lift] = new_values[i]

  one_rep_max_lifts = json.dumps(one_rep_max_lifts)
 
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET one_rep_maxes=%s WHERE email=%s", (one_rep_max_lifts, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET one_rep_maxes=? WHERE email=?", (one_rep_max_lifts, email,))

# updates reps and weight
def update_lifts_for_reps(new_lifts_for_reps, db_path=db_path):
  email = logged_in_user_email(db_path)
  lifts_for_reps = json.loads(fetch_lifts_for_reps(db_path))
  new_lifts_for_reps = list(new_lifts_for_reps.values())
  
  for i, lift in enumerate(lifts_for_reps.keys()):
    lifts_for_reps[lift] = new_lifts_for_reps[i]

  lifts_for_reps = json.dumps(lifts_for_reps)

  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET lifts_for_reps=%s WHERE email=%s", (lifts_for_reps, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET lifts_for_reps=? WHERE email=?", (lifts_for_reps, email,))

def update_lift_history(lift_history, db_path=db_path):
  email = logged_in_user_email(db_path)
  current_lift_history = fetch_lift_history(db_path)
  lift_history = [[exercise, value] for exercise, value in lift_history.items()]
  new_lift_history = None
  if current_lift_history == None:
    indices = list(reversed(range(len(lift_history))))
    for i, lift in enumerate(lift_history):
      lift.append(indices[i])
    new_lift_history = json.dumps(lift_history)
  else:
    last_index = json.loads(current_lift_history)[0][-1]+1
    current_lift_history = list(reversed(json.loads(current_lift_history)))
    for lift in reversed(lift_history):
      lift.append(last_index)
      current_lift_history.append(lift)
      last_index += 1
    new_lift_history = json.dumps(list(reversed(current_lift_history)))
  
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET lift_history=%s WHERE email=%s", (new_lift_history, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET lift_history=? WHERE email=?", (new_lift_history, email,))

def update_big_lifts_units(db_path=db_path):
  email = logged_in_user_email(db_path)
  units = fetch_units(db_path)
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET units=%s WHERE email=%s", (units, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET units=? WHERE email=?", (units, email,))

def clear_one_rep_maxes():
  email = logged_in_user_email()
  one_rep_maxes = json.loads(fetch_one_rep_maxes())
  for exercise, value in one_rep_maxes.items():
    one_rep_maxes[exercise] = "0"

  one_rep_maxes = json.dumps(one_rep_maxes)
  
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET one_rep_maxes=%s WHERE email=%s", (one_rep_maxes, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET one_rep_maxes=? WHERE email=?", (one_rep_maxes, email,))

def clear_lifts_for_reps():
  email = logged_in_user_email()
  lifts_for_reps = json.loads(fetch_lifts_for_reps())
  for exercise, values in lifts_for_reps.items():
    lifts_for_reps[exercise] = ["0", "0"]

  lifts_for_reps = json.dumps(lifts_for_reps)

  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET lifts_for_reps=%s WHERE email=%s", (lift_for_reps, email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET lifts_for_reps=? WHERE email=?", (lift_for_reps, email,))

def sort_exercises(exercise):
  if exercise in ["Bench Press", "Incline Bench Press"]: return 4
  elif exercise in ["Deadlift", "Sumo Deadlift"]: return 3
  elif exercise in ["Back Squat", "Front Squat"]: return 2
  elif exercise in ["Overhead Press", "Push Press"]: return 1 

# returns sorted dictionary containing updated lifts
def lift_difference(new_lifts, db_path=db_path, one_RM=False, lifts_reps=False):
  difference = None
  if one_RM:
    db_lifts = set(": ".join([exercise, weight]) for exercise, weight in json.loads(fetch_one_rep_maxes(db_path)).items())
    new_lifts = set(": ".join([exercise, weight]) for exercise, weight in new_lifts.items() if not weight == '0.0')
    diff = list(new_lifts.difference(db_lifts)) # local lifts that are not in db
    difference = {exercise.split(": ")[0]:exercise.split(": ")[1] for exercise in diff}
  elif lifts_reps:
    db_lifts = set(":".join([exercise, "x".join(values)]) for exercise, values in json.loads(fetch_lifts_for_reps(db_path)).items())
    new_lifts = set(":".join([exercise, "x".join(values)]) for exercise, values in new_lifts.items() if not values[1] == '0.0')
    diff = list(new_lifts.difference(db_lifts))
    difference = {exercise.split(":")[0]:exercise.split(":")[1].split("x") for exercise in diff}
  return {key: value for key, value in sorted(difference.items(), key=lambda exercise: sort_exercises(exercise[0]))}

def delete_history_entry(entry_index, db_path=db_path):
  email = logged_in_user_email(db_path)
  lift_history = json.loads(fetch_lift_history(db_path))
  lift_history = [lift for lift in lift_history if not lift[-1] == entry_index]
  
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      if not len(lift_history) == 0:
        cursor.execute("UPDATE big_lifts SET lift_history=%s WHERE email=%s", (json.dumps(lift_history), email,))
      else:
        cursor.execute("UPDATE big_lifts SET lift_history=NULL WHERE email=%s", (email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    if not len(lift_history) == 0:
      cursor.execute("UPDATE 'big_lifts' SET lift_history=? WHERE email=?", (json.dumps(lift_history), email,))
    else:
      cursor.execute("UPDATE 'big_lifts' SET lift_history=NULL WHERE email=?", (email,))

def convert_lift_history_weight(convert_to_units, db_path=db_path):
  email = logged_in_user_email(db_path)
  try:
    lift_history = json.loads(fetch_lift_history(db_path))
  except TypeError: # lift history is empty
    return
  if convert_to_units == "kg":
    for lift in lift_history:
      if isinstance(lift[1], list): # second element of history entry is list, it is lifts_for_reps entry
        lift[1][1] = str(pounds_to_kg(float(lift[1][1])))
      else:
        lift[1] = str(pounds_to_kg(float(lift[1])))
  elif convert_to_units == "lb":
    for lift in lift_history:
      if isinstance(lift[1], list):
        lift[1][1] = str(kg_to_pounds(float(lift[1][1])))
      else:
        lift[1] = str(kg_to_pounds(float(lift[1])))
  
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE big_lifts SET lift_history=%s WHERE email=%s", (json.dumps(lift_history), email,))

  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'big_lifts' SET lift_history=? WHERE email=?", (json.dumps(lift_history), email,))



import os
from PyQt5.QtWidgets import (QWidget, QGridLayout, QFrame, QLabel, QProgressBar,
                             QPushButton, QFrame, QHBoxLayout, QVBoxLayout,
                             QTableWidget, QTableWidgetItem, QHeaderView, QAbstractItemView)
from PyQt5.QtGui import QFont, QCursor, QIcon
from PyQt5.QtCore import Qt, QSize, pyqtSlot
from fitness_tracker.user_profile.profile_db import fetch_units, fetch_user_weight, fetch_goal_weight
from .nutrition_db import table_exists, create_nutrition_table, fetch_nutrition_data, fetch_calorie_goal
from .change_weight_dialog import ChangeWeightDialog

path = os.path.abspath(os.path.dirname(__file__))
icons_path = os.path.join(path, "icons")

icons = {"pencil": os.path.join(icons_path, "pencil.png"),
         "plus": os.path.join(icons_path, "plus.png"),
         "left": os.path.join(icons_path, "left.png"),
         "right": os.path.join(icons_path, "right.png")}

class NotesPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    if not table_exists():
      create_nutrition_table()
      fetch_nutrition_data()
    self.setStyleSheet(   
    """QWidget{
      color:#c7c7c7;
      font-weight: bold;
      font-family: "Ubuntu";
      font-size: 16px;
      }
      """)
    self.units = "kg" if fetch_units() == "metric" else "lb"
    self.user_weight = fetch_user_weight()
    self.user_goal_weight = fetch_goal_weight()
    self.calorie_goal = fetch_calorie_goal()
    self.change_weight_dialog = ChangeWeightDialog()
    self.change_weight_dialog.change_current_weight_signal.connect(lambda weight: self.change_current_weight(weight))
    self.change_weight_dialog.change_goal_weight_signal.connect(lambda weight: self.change_goal_weight(weight))
    self.change_weight_dialog.change_calorie_goal_signal.connect(lambda calorie_goal: self.change_calorie_goal(calorie_goal))
    self.create_panel()
    self.setStyleSheet("QLabel{color:white;}")
  
  @pyqtSlot(str)
  def change_current_weight(self, weight):
    self.current_weight_label.setText(" ".join(["Current Weight:", weight, self.units]))
  
  @pyqtSlot(str)
  def change_goal_weight(self, weight):
    self.goal_weight_label.setText(" ".join(["Current Weight:", weight, self.units]))

  @pyqtSlot(str)
  def change_calorie_goal(self, calorie_goal):
    self.calorie_goal_label.setText(" ".join(["Calorie Goal: \n", calorie_goal]))
    self.progress_bar.setMaximum(int(calorie_goal))
    calories_left = self.progress_bar.maximum() - self.progress_bar.value()
    self.progress_bar.setFormat(" ".join([str(calories_left), "calories left"]))

  def create_panel(self):
    grid = QGridLayout()
    grid.setContentsMargins(2, 0, 8, 8)
    grid.addWidget(self.create_description(), 0, 0, 1, 3)
    grid.addLayout(self.create_stats(), 6, 0, 6, 1)
    grid.addWidget(self.create_nutrition_summary(), 6, 1, 6, 1)
    grid.addWidget(self.create_weight_panel(), 7, 2, 5, 1)
    grid.addLayout(self.create_notes(), 16, 0, 10, 3)
    self.setLayout(grid)

  def create_description(self):
    self.description = QLabel("Improve your diet and eating habits by tracking what you eat and searching through our database for healthy \nfoods.")
    return self.description

  def create_nutrition_summary(self):
    nsummary_layout = QVBoxLayout()
    daily_monthlyavg_buttons = QHBoxLayout()
    daily_button = QPushButton("Daily")
    monthly_button = QPushButton("Monthly")
    daily_monthlyavg_buttons.addWidget(daily_button)
    daily_monthlyavg_buttons.addWidget(monthly_button)

    nsummary_layout.addLayout(daily_monthlyavg_buttons)
    
    temp_label = QLabel("Proteins: 30g \nCarbs: 300g \nCalories: 2000kcal")
    nsummary_layout.addWidget(temp_label) # Temp

    nsummary_layout_framed = QFrame()
    nsummary_layout_framed.setLayout(nsummary_layout)
    nsummary_layout_framed.setFrameStyle(QFrame.Box)
    nsummary_layout_framed.setLineWidth(3)
    nsummary_layout_framed.setObjectName("frame")
    nsummary_layout_framed.setStyleSheet("""#frame {color: #322d2d;}""")
    return nsummary_layout_framed

  def create_stats(self):
    stats_layout = QHBoxLayout()
    
    calorie_goal_layout = QVBoxLayout()
    calorie_goal_frame = QFrame()
    self.dci_label = QLabel("Daily Calorie Intake")
    self.dci_label.setAlignment(Qt.AlignCenter)
    self.calorie_goal_label = QLabel(" ".join(["Daily Goal: ", self.calorie_goal, "kcal"]))
    #self.calorie_goal_label.setFixedHeight(40)
    self.calorie_goal_label.setFont(QFont("Ariel", 15))
    self.calorie_goal_label.setAlignment(Qt.AlignCenter)
     
    self.progress_bar = QProgressBar()
    self.progress_bar.setStyleSheet("background-color:grey;")
    self.progress_bar.setMaximum(int(self.calorie_goal))
    self.progress_bar.setValue(500)
    calories_left = self.progress_bar.maximum() - self.progress_bar.value()
    self.progress_bar.setFormat("")
    self.progress_bar.setAlignment(Qt.AlignCenter)
    self.progress_bar.setMaximumHeight(12)
    self.calorie_label = QLabel(str(calories_left) + " calories left from goal")
    self.calorie_label.setAlignment(Qt.AlignCenter)

    intake_button_layout = QHBoxLayout()
    self.calculate_intake = QPushButton("Calculate Daily Intake")
    self.edit_intake = QPushButton("Edit Daily Intake")
    intake_button_layout.addWidget(self.calculate_intake)
    intake_button_layout.addWidget(self.edit_intake)

    calorie_goal_layout.addWidget(self.dci_label)
    calorie_goal_layout.addWidget(self.progress_bar)
    calorie_goal_layout.addWidget(self.calorie_label)
    calorie_goal_layout.addWidget(self.calorie_goal_label)
    calorie_goal_layout.addLayout(intake_button_layout)

    calorie_goal_frame.setLayout(calorie_goal_layout)
    calorie_goal_frame.setFrameStyle(QFrame.Box)
    calorie_goal_frame.setLineWidth(3)
    calorie_goal_frame.setObjectName("frame")
    calorie_goal_frame.setStyleSheet("""#frame {color: #322d2d;}""")
    stats_layout.addWidget(calorie_goal_frame)

    return stats_layout

  def create_weight_panel(self):
    current_weight_layout = QHBoxLayout()
    self.current_weight_label = QLabel(" ".join(["Current Weight:", self.user_weight, self.units]))
    self.current_weight_label.setFont(QFont("Ariel", 15))
    edit_current_weight_button = QPushButton()
    edit_current_weight_button.setFlat(True)
    edit_current_weight_button.setIcon(QIcon(icons["pencil"]))
    edit_current_weight_button.setIconSize(QSize(16, 16))
    edit_current_weight_button.setCursor(QCursor(Qt.PointingHandCursor))
    edit_current_weight_button.clicked.connect(lambda: self.change_weight_dialog.show())
    current_weight_layout.addWidget(self.current_weight_label)
    current_weight_layout.addWidget(edit_current_weight_button)

    weight_layout = QVBoxLayout()
    weight_frame = QFrame()
    weight_frame.setFrameStyle(QFrame.Box)
    weight_frame.setLineWidth(3)
    weight_frame.setObjectName("frame")
    weight_frame.setStyleSheet("""#frame {color: #322d2d;}""")
    goal_weight_layout = QHBoxLayout()
    self.goal_weight_label = QLabel(" ".join(["Goal Weight:", self.user_goal_weight, self.units]))
    self.goal_weight_label.setFont(QFont("Ariel", 15))
    edit_goal_weight_button = QPushButton()
    edit_goal_weight_button.setFlat(True)
    edit_goal_weight_button.setIcon(QIcon(icons["pencil"]))
    edit_goal_weight_button.setIconSize(QSize(16, 16))
    edit_goal_weight_button.setCursor(QCursor(Qt.PointingHandCursor))
    edit_goal_weight_button.clicked.connect(lambda: self.change_weight_dialog.show())
    goal_weight_layout.addWidget(self.goal_weight_label)
    goal_weight_layout.addWidget(edit_goal_weight_button)

    weight_layout.addLayout(current_weight_layout)
    weight_layout.addLayout(goal_weight_layout)
    weight_frame.setLayout(weight_layout)
    return weight_frame

  def create_notes(self):
    
    table = QTableWidget(2, 4)
    table.setStyleSheet("background-color: #603A40;color:white;")
    table.setEditTriggers(QAbstractItemView.NoEditTriggers)
    table.verticalHeader().setVisible(False)
    
    meals = ["Breakfast", "Lunch", "Dinner", "Snacks"]
    
    plus_button = QPushButton(QIcon(icons["plus"]), "Add Food", self)
    plus_button1 = QPushButton(QIcon(icons["plus"]), "Add Food", self)
    plus_button2 = QPushButton(QIcon(icons["plus"]), "Add Food", self)
    plus_button3 = QPushButton(QIcon(icons["plus"]), "Add Food", self)
    
    buttons = [plus_button, plus_button1, plus_button2, plus_button3]
    j = i = 0
    for item in meals:
      buttons[i].setFlat(True)
      table.setHorizontalHeaderItem(i, QTableWidgetItem(item))
      table.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
      table.setCellWidget(0, j, buttons[i])
      j += 1
      i += 1

    for i in range(2):
      table.verticalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
    
    label = QLabel("Monday, 31.2.2005", self)
    label.setAlignment(Qt.AlignCenter)
    
    table_title_layout = QHBoxLayout()
    
    left_button = QPushButton(QIcon(icons["left"]), "", self)
    left_button.setFlat(True)
    left_button.setFixedWidth(20)
    left_button.setCursor(QCursor(Qt.PointingHandCursor))
    right_button = QPushButton(QIcon(icons["right"]), "",self)
    right_button.setFlat(True)
    right_button.setFixedWidth(20)
    right_button.setCursor(QCursor(Qt.PointingHandCursor))

    table_title_layout.addWidget(left_button)
    table_title_layout.addWidget(label)
    table_title_layout.addWidget(right_button)
    
    title_frame = QFrame()
    title_frame.setStyleSheet("background-color: #603A40;")
    title_frame.setFrameStyle(QFrame.StyledPanel)
    title_frame.setLayout(table_title_layout)

    table_wrapper = QVBoxLayout()
    table_wrapper.addWidget(table)
    
    grid = QGridLayout()
    grid.addWidget(title_frame, 0, 0)
    grid.addLayout(table_wrapper, 1, 0)
    
    return grid



from PyQt5.QtWidgets import QWidget, QGridLayout
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot
from .header import Header
from .side_panel import SidePanel
from .main_panel import MainPanel

class Homepage(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
     super().__init__()
     #self.header = Header(self, "Fitness Tracker  ")
     self.side_panel = SidePanel(self)
     self.side_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name))
     self.main_panel = MainPanel(self)
     self.create_grid()

  def create_grid(self):
    grid = QGridLayout()
    grid.setContentsMargins(0, 0, 0, 0)
    #grid.addWidget(self.header, 0, 1, 1, 4)
    grid.addWidget(self.side_panel, 1, 1, 8, 1)
    grid.addWidget(self.main_panel, 1, 2, 8, 3)
    self.setLayout(grid)
  
  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)


from PyQt5.QtWidgets import QWidget, QGridLayout
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from .main_panel import MainPanel

class Signup(QWidget):
  display_layout_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.main_panel = MainPanel(self)
    self.main_panel.emit_layout_name.connect(lambda layout_name: self.emit_display_layout_signal(layout_name)) 
    self.create_grid()

  def create_grid(self):
    grid = QGridLayout()
    grid.addWidget(self.main_panel, 0, 0, 8, 4)
    self.setLayout(grid)

  @pyqtSlot(str)
  def emit_display_layout_signal(self, layout_name):
    self.display_layout_signal.emit(layout_name)



import math

def pounds_to_kg(lbs):
  return round(float(lbs)/2.2046, 2)

def kg_to_pounds(kg):
  return round(float(kg)*2.2046, 2)

def inch_to_cm(inch):
  return round(float(inch)*2.54, 2)

def cm_to_inch(cm):
  return round(float(cm)/2.54, 2)

def metric_to_imperial_height(cm):
  inches_total = cm_to_inch(float(cm))
  feet = math.floor(inches_total/12)
  inches = round(inches_total - 12*feet)
  return feet, inches

def imperial_to_metric_height(feet, inches):
  feet_to_cm = inch_to_cm(float(feet)*12)
  inches_to_cm = inch_to_cm(float(inches))
  return math.ceil(feet_to_cm+inches_to_cm)



from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QHBoxLayout, QVBoxLayout, QComboBox, QFrame
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt

class MainPanel(QWidget):
  def __init__(self, parent):
    super().__init__(parent)
    self.create_panel()
  
  def create_panel(self): 
    grid = QGridLayout()
    grid.addWidget(self.create_graph_layout(), 0, 0 , 3, 2)
    grid.addWidget(self.create_max_layout(), 4, 0, 1, 1)
    grid.addWidget(self.create_strongest_layout(), 5, 0, 1, 1)
    grid.addWidget(self.create_general_layout(), 4, 1, 2, 1)
    self.setLayout(grid)
  
  def create_graph_layout(self):
    graph_layout = QVBoxLayout()
    
    graph = QWidget(self)
    graph.setStyleSheet("background-color: red")
    
    graph_dropdowns_layout = QHBoxLayout()
    
    time_period_dropdown = QComboBox()
    time_period_dropdown.addItems(["Daily", "Weekly", "Monthly", "Yearly"])

    exercise_dropdown = QComboBox()
    exercise_dropdown.addItems(["Deadlift", "Bench", "Squat", "Military Press", "Barbell Rows"])
    
    graph_dropdowns_layout.addWidget(time_period_dropdown)
    graph_dropdowns_layout.addWidget(exercise_dropdown)

    graph_layout.addWidget(graph)
    graph_layout.addLayout(graph_dropdowns_layout)
   
    graph_frame = QFrame()
    graph_frame.setFrameStyle(QFrame.StyledPanel)
    graph_frame.setLayout(graph_layout)
 
    return graph_frame
  
  def create_max_layout(self):
    max_layout = QVBoxLayout()
     
    one_rep_label = QLabel("One Rep Max", self)
    one_rep_label.setFont(QFont("Ariel", 15))
    one_rep_label.setAlignment(Qt.AlignLeft)
    one_rep_label.setFixedSize(135, 25)
     
    max_bench = QLabel("Bench: 80kg", self)
    max_bench.setAlignment(Qt.AlignLeft)
    max_bench.setFixedSize(120, 20)

    max_deadlift = QLabel("Deadlift: 180kg", self)
    max_deadlift.setAlignment(Qt.AlignLeft)
    max_deadlift.setFixedSize(120, 20)

    max_squat = QLabel("Squat: 80kg", self)
    max_squat.setAlignment(Qt.AlignLeft)
    max_squat.setFixedSize(120, 20)

    max_layout.addWidget(one_rep_label)
    max_layout.addWidget(max_bench)
    max_layout.addWidget(max_deadlift)
    max_layout.addWidget(max_squat)
    
    max_frame = QFrame()
    max_frame.setFrameStyle(QFrame.StyledPanel)
    max_frame.setLayout(max_layout) 
    
    return max_frame

  def create_strongest_layout(self):
    strongest_layout = QVBoxLayout()

    strongest_lift = QLabel("Strongest lift: Deadlift", self)
    strongest_lift.setAlignment(Qt.AlignLeft)
    strongest_lift.setFixedSize(200, 20)

    weakest_lift = QLabel("Weakest lift: Squat", self)
    weakest_lift.setAlignment(Qt.AlignLeft)
    weakest_lift.setFixedSize(200, 20)

    strongest_layout.addWidget(strongest_lift)
    strongest_layout.addWidget(weakest_lift)

    strongest_frame = QFrame()
    strongest_frame.setFrameStyle(QFrame.StyledPanel)
    
    strongest_frame.setLayout(strongest_layout)
    
    return strongest_frame

  def create_general_layout(self):
    general_layout = QVBoxLayout()
    
    general_label = QLabel("General", self)
    general_label.setFont(QFont("Ariel", 15))
    general_label.setAlignment(Qt.AlignLeft)
    general_label.setFixedSize(135, 25)
     
    stats_one = QLabel("Chin up count: 10", self)
    stats_one.setAlignment(Qt.AlignLeft)
    stats_one.setFixedSize(200, 20)

    stats_two = QLabel("Dip count: 12", self)
    stats_two.setAlignment(Qt.AlignLeft)
    stats_two.setFixedSize(200, 20)

    stats_three = QLabel("Last updated physique: 4 days ago", self)
    stats_three.setAlignment(Qt.AlignLeft)
    stats_three.setFixedSize(250, 20)
    
    general_layout.addWidget(general_label)
    general_layout.addWidget(stats_one)
    general_layout.addWidget(stats_two)
    general_layout.addWidget(stats_three)

    general_frame = QFrame()
    general_frame.setFrameStyle(QFrame.StyledPanel)
    general_frame.setLayout(general_layout) 

    return general_frame



import os
from PyQt5.QtWidgets import QWidget, QLabel, QGridLayout, QPushButton, QVBoxLayout
from PyQt5.QtCore import Qt, QSize, QPoint, QDir, pyqtSignal
from PyQt5.QtGui import QFont, QIcon, QCursor, QPainter, QColor, QBrush, QFontDatabase

path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "icons")

icons = {"Home": os.path.join(path, "home.png"),
         "Profile": os.path.join(path, "user.png"),
         "Logout": os.path.join(path, "logout.png"),
         "Big Lifts": os.path.join(path, "big_lifts.png"),
         "Workouts": os.path.join(path, "workouts.png"),
         "Nutrition": os.path.join(path, "calories.png"),
         "Weight Loss Notes": os.path.join(path, "weight_loss.png"),
         "1 Rep Max": os.path.join(path, "one_rep_max.png"),
         "Body Fat": os.path.join(path, "body_fat.png"),
         "Strength Estimator": os.path.join(path, "strength_es.png"),
         "Strength": os.path.join(path, "strength.png"),
         "Weight Loss": os.path.join(path, "weight_loss.png")}

icon_size = QSize(24, 24)

class SidePanel(QWidget):
  emit_layout_name = pyqtSignal(str)

  def __init__(self, parent):
    super().__init__(parent)
    self.create_panel()
    self.setMaximumWidth(201)
    QFontDatabase.addApplicationFont("font/Ubuntu-Regular.ttf")
    self.setStyleSheet("""
                       QPushButton{
                         text-align: center;
                         background-color: #440D0F;
                         border-radius: 3px;
                         min-height: 23px;
                         min-width: 180px;
                         color: #c7c7c7;
                         font: 16px;
                         font-weight:500;
                         font-family:Ubuntu;
                         padding-bottom: 3px;
                         }
                       QPushButton:hover:!pressed{
                         background-color: #5D1A1D
                       }
                       QPushButton:pressed{
                         background-color: #551812
                       }
                       """)
  
  def create_panel(self):
    line_divider_general = self.create_line_divider()
    line_divider_calc = self.create_line_divider()
    line_divider_notes = self.create_line_divider()
    line_divider_stats = self.create_line_divider()

    general_layout = QVBoxLayout()
    general_layout.setContentsMargins(0, 0, 0, 0)
    home_button = QPushButton("Home", self)
    home_button.clicked.connect(lambda: self.emit_layout_name.emit(home_button.text()))
    user_button = QPushButton("Profile", self)
    user_button.clicked.connect(lambda: self.emit_layout_name.emit(user_button.text()))
    logout_button = QPushButton("Logout", self)
    logout_button.clicked.connect(lambda: self.emit_layout_name.emit(logout_button.text()))

    notes_layout = QVBoxLayout()
    notes_layout.setContentsMargins(0, 0, 0, 0)
    lifts_button = QPushButton("Big Lifts", self)
    lifts_button.clicked.connect(lambda: self.emit_layout_name.emit(lifts_button.text()))
    workouts_button = QPushButton("Workouts", self)
    workouts_button.clicked.connect(lambda: self.emit_layout_name.emit(workouts_button.text()))
    nutrition_button = QPushButton("Nutrition", self)
    nutrition_button.clicked.connect(lambda: self.emit_layout_name.emit(nutrition_button.text()))
    weight_notes_button = QPushButton("Weight Loss Notes", self)
    weight_notes_button.clicked.connect(lambda: self.emit_layout_name.emit(weight_notes_button.text()))
    
    calculators_layout = QVBoxLayout()
    calculators_layout.setContentsMargins(0, 0, 0, 0)
    max_button = QPushButton("1 Rep Max Calculator", self)
    max_button.clicked.connect(lambda: self.emit_layout_name.emit(max_button.text()))
    fat_button = QPushButton("Body Fat Calculator", self)
    fat_button.clicked.connect(lambda: self.emit_layout_name.emit(fat_button.text()))
    strength_es_button = QPushButton("Strength Estimator", self)
    strength_es_button.clicked.connect(lambda: self.emit_layout_name.emit(strength_es_button.text()))
    
    stats_layout = QVBoxLayout()
    stats_layout.setContentsMargins(0, 0, 0, 0)
    strength_button = QPushButton("Strength Statistics", self)
    strength_button.clicked.connect(lambda: self.emit_layout_name.emit(strength_button.text()))
    weight_loss_button = QPushButton("Weight Loss Stats", self)
    weight_loss_button.clicked.connect(lambda: self.emit_layout_name.emit(weight_loss_button.text()))
    
    layouts = {general_layout: [line_divider_general, home_button, user_button, logout_button],
               notes_layout: [line_divider_notes, lifts_button, workouts_button, nutrition_button, weight_notes_button],
               calculators_layout: [line_divider_calc, max_button, fat_button, strength_es_button],
               stats_layout: [line_divider_stats, strength_button, weight_loss_button]}
    
    grid = QGridLayout()
    self.setLayout(grid)
    
    for i, (key, value) in enumerate(layouts.items()):
      for widget in value:
        key.addWidget(widget)
        if isinstance(widget, QPushButton):
          widget.setIconSize(icon_size)
          widget.setCursor(QCursor(Qt.PointingHandCursor))
      grid.addLayout(key, i, 0)

  def create_line_divider(self):
    line_divider = QWidget()
    line_divider.setStyleSheet("""
    background-color: #603A36;
    """)
    line_divider.setMaximumSize(183, 3)
    return line_divider

  def paintEvent(self, event):
    painter = QPainter()
    painter.begin(self)
    painter.setRenderHint(QPainter.Antialiasing)
    painter.fillRect(event.rect(), QBrush(QColor(17, 16, 15, 255)))


import json
import os
from PyQt5.QtWidgets import QLabel, QWidget, QPushButton, QFormLayout, QLineEdit, QHBoxLayout
from PyQt5.QtCore import pyqtSignal
from .big_lifts_db import (fetch_preferred_lifts, fetch_lifts_for_reps, lift_difference,
                           update_lift_history, update_lifts_for_reps)
from fitness_tracker.user_profile.profile_db import fetch_units

class UpdateLiftsForRepsWindow(QWidget):
  change_lifts_for_reps_signal = pyqtSignal(bool)
  history_signal = pyqtSignal(bool)

  def __init__(self):
    super().__init__()
    self.units = "kg" if fetch_units() == "metric" else "lb"
    self.preferred_lifts = json.loads(fetch_preferred_lifts())
    self.setWindowTitle("Update Lifts For Reps")
    self.setLayout(self.create_panel())
    self.set_line_edit_values()

  def create_panel(self):
    form_layout = QFormLayout()
    
    exercise_label = QLabel("Exercise")
    header_layout = QHBoxLayout()
    reps_label = QLabel("Reps")
    weight_label = QLabel("Weight")
    header_layout.addWidget(reps_label)
    header_layout.addWidget(weight_label)

    horizontal_press_label = QLabel(self.preferred_lifts["Horizontal Press"])
    self.horizontal_press_reps_edit = QLineEdit()
    x_label = QLabel("x")
    self.horizontal_press_edit = QLineEdit() 
    units_label = QLabel(self.units)
    hbox = QHBoxLayout()
    hbox.addWidget(self.horizontal_press_reps_edit)
    hbox.addWidget(x_label)
    hbox.addWidget(self.horizontal_press_edit)
    hbox.addWidget(units_label)

    floor_pull_label = QLabel(self.preferred_lifts["Floor Pull"])
    self.floor_pull_reps_edit = QLineEdit()
    x_label1 = QLabel("x")
    self.floor_pull_edit = QLineEdit()
    units_label1 = QLabel(self.units)
    hbox1 = QHBoxLayout()
    hbox1.addWidget(self.floor_pull_reps_edit)
    hbox1.addWidget(x_label1)
    hbox1.addWidget(self.floor_pull_edit)
    hbox1.addWidget(units_label1)

    squat_label = QLabel(self.preferred_lifts["Squat"])
    self.squat_reps_edit = QLineEdit()
    x_label2 = QLabel("x")
    self.squat_edit = QLineEdit()
    units_label2 = QLabel(self.units)
    hbox2 = QHBoxLayout()
    hbox2.addWidget(self.squat_reps_edit)
    hbox2.addWidget(x_label2)
    hbox2.addWidget(self.squat_edit)
    hbox2.addWidget(units_label2)

    vertical_press_label = QLabel("Overhead Press")
    self.vertical_press_reps_edit = QLineEdit()
    x_label3 = QLabel("x")
    self.vertical_press_edit = QLineEdit()
    units_label3 = QLabel(self.units)
    hbox3 = QHBoxLayout()
    hbox3.addWidget(self.vertical_press_reps_edit)
    hbox3.addWidget(x_label3)
    hbox3.addWidget(self.vertical_press_edit)
    hbox3.addWidget(units_label3)

    save_button = QPushButton("Save")
    save_button.clicked.connect(lambda: self.save_lifts_for_reps())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close_update_lifts_for_reps())
    
    form_layout.addRow(exercise_label, header_layout)
    form_layout.addRow(horizontal_press_label, hbox)
    form_layout.addRow(floor_pull_label, hbox1)
    form_layout.addRow(squat_label, hbox2)
    form_layout.addRow(vertical_press_label, hbox3)
    form_layout.addRow(save_button, cancel_button)
    return form_layout

  def save_lifts_for_reps(self):
    try:
      exercises = list(json.loads(fetch_lifts_for_reps()).keys())
      horizontal_press_weight = str(float(self.horizontal_press_edit.text()))
      floor_pull_weight = str(float(self.floor_pull_edit.text()))
      squat_weight = str(float(self.squat_edit.text()))
      vertical_press_weight = str(float(self.vertical_press_edit.text()))

      horizontal_press_reps = str(int(self.horizontal_press_reps_edit.text()))      
      floor_pull_reps = str(int(self.floor_pull_reps_edit.text()))
      squat_reps = str(int(self.squat_reps_edit.text()))
      vertical_press_reps = str(int(self.vertical_press_reps_edit.text()))
    
      new_lifts_for_reps = {exercises[0]: [horizontal_press_reps, horizontal_press_weight],
                            exercises[1]: [floor_pull_reps, floor_pull_weight],
                            exercises[2]: [squat_reps, squat_weight],
                            exercises[3]: [vertical_press_reps, vertical_press_weight]}
      diff = lift_difference(new_lifts_for_reps, lifts_reps=True)
      update_lift_history(diff)
      self.history_signal.emit(True)
      update_lifts_for_reps(new_lifts_for_reps)
      self.change_lifts_for_reps_signal.emit(True)
      self.set_line_edit_values()
      self.close()
    except ValueError:
      pass
  
  def close_update_lifts_for_reps(self):
    self.close()
    self.set_line_edit_values()

  def set_line_edit_values(self):
    lift_values = list(json.loads(fetch_lifts_for_reps()).values())
    reps = [lift[0] for lift in lift_values]
    weight = [lift[1] for lift in lift_values]
    
    reps_line_edit = [self.horizontal_press_reps_edit,
                      self.floor_pull_reps_edit,
                      self.squat_reps_edit,
                      self.vertical_press_reps_edit]

    weight_line_edit = [self.horizontal_press_edit,
                        self.floor_pull_edit,
                        self.squat_edit,
                        self.vertical_press_edit]

    for i, line_edit in enumerate(reps_line_edit):
      line_edit.setText(reps[i])

    for i, line_edit in enumerate(weight_line_edit):
      line_edit.setText(weight[i])



from PyQt5.QtWidgets import QWidget, QGridLayout, QHBoxLayout, QVBoxLayout, QLabel, QGroupBox, QRadioButton, QFrame
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt

class Header(QWidget):
  def __init__(self, parent, title):
    super().__init__(parent)
    self.title = title
    self.create_header()
  
  def create_header(self):
    grid = QGridLayout()
    self.setLayout(grid)
    
    title_layout = QHBoxLayout()
    title_label = QLabel(self.title, self)
    title_label.setAlignment(Qt.AlignCenter)
    title_label.setFont(QFont("Arial", 25))
    title_layout.addWidget(title_label)

    header_frame = QFrame()
    header_frame.setFrameStyle(QFrame.StyledPanel)
    header_layout = QHBoxLayout()

    current_weight_frame = QFrame()
    current_weight_frame.setFrameStyle(QFrame.StyledPanel)
    current_weight_layout = QVBoxLayout()
    
    current_weight_label = QLabel("Current Weight", self)
    current_weight_label.setAlignment(Qt.AlignCenter)
    current_weight_label2 = QLabel("90.0 kg", self)
    current_weight_label2.setAlignment(Qt.AlignCenter)

    current_weight_layout.addWidget(current_weight_label)
    current_weight_layout.addWidget(current_weight_label2)

    current_weight_frame.setLayout(current_weight_layout)
    
    desired_weight_frame = QFrame()
    desired_weight_frame.setFrameStyle(QFrame.StyledPanel)
    desired_weight_layout = QVBoxLayout()

    desired_weight_label = QLabel("Desired Weight", self)
    desired_weight_label.setAlignment(Qt.AlignCenter)
    desired_weight_label2 = QLabel("85.0 kg", self)
    desired_weight_label2.setAlignment(Qt.AlignCenter)

    desired_weight_layout.addWidget(desired_weight_label)
    desired_weight_layout.addWidget(desired_weight_label2)
    desired_weight_frame.setLayout(desired_weight_layout) 
    
    bmi_frame = QFrame()
    bmi_frame.setFrameStyle(QFrame.StyledPanel)
    bmi_layout = QVBoxLayout()

    bmi_label = QLabel("BMI", self)
    bmi_label.setAlignment(Qt.AlignCenter)
    bmi_label2 = QLabel("25.8", self)
    bmi_label2.setAlignment(Qt.AlignCenter)
    
    bmi_layout.addWidget(bmi_label)
    bmi_layout.addWidget(bmi_label2)
    bmi_frame.setLayout(bmi_layout)

    body_fat_frame = QFrame()
    body_fat_frame.setFrameStyle(QFrame.StyledPanel)
    body_fat_layout = QVBoxLayout()

    body_fat_label = QLabel("Body Fat", self)
    body_fat_label.setAlignment(Qt.AlignCenter)
    body_fat_label2 = QLabel("25.7%", self)
    body_fat_label2.setAlignment(Qt.AlignCenter)

    body_fat_layout.addWidget(body_fat_label)
    body_fat_layout.addWidget(body_fat_label2)
    body_fat_frame.setLayout(body_fat_layout)
    
    units_frame = QFrame()
    units_frame.setFrameStyle(QFrame.StyledPanel)
    units_layout = QVBoxLayout()
    units_group = QGroupBox()
    units_group.setFixedHeight(100)
    m_radio = QRadioButton("Metric")
    m_radio.setChecked(True)
    i_radio = QRadioButton("Imperial")
     
    units_layout.addWidget(m_radio)
    units_layout.addWidget(i_radio)

    units_group.setLayout(units_layout)

    groupbox_layout = QHBoxLayout()
    groupbox_layout.addWidget(units_group)
    units_frame.setLayout(groupbox_layout)

    header_layout.addWidget(current_weight_frame)
    header_layout.addWidget(desired_weight_frame)
    header_layout.addWidget(bmi_frame)
    header_layout.addWidget(body_fat_frame)
    header_layout.addWidget(units_frame)
    header_frame.setLayout(header_layout)
    grid.addLayout(title_layout, 0, 0, 1, 1)
    grid.addWidget(header_frame, 0, 3, 1, 3)



import requests
import os
import shutil

class Spoonacular:
  def __init__(self, api_key):
    self.base_api_url = "https://api.spoonacular.com/"
    self.api_key = api_key
  
  def make_request(self, path, params):
    uri = self.base_api_url + path
    params["apiKey"] = self.api_key
    request = requests.get(uri, params=params)
    return request

  def complex_recipe_search(self, query, **kwargs):
    endpoint = "recipes/complexSearch"
    params = {"query": query, **kwargs}
    return self.make_request(endpoint, params)

  def search_recipes_by_nutrients(self, **kwargs):
    endpoint = "recipes/findByNutrients"
    params = {**kwargs}
    return self.make_request(endpoint, params)

  def search_recipes_by_ingredients(self, ingredients, **kwargs):
    endpoint = "recipes/findByIngredients"
    params = {"ingredients": ingredients, **kwargs}
    return self.make_request(endpoint, params)

  def get_recipe_information(self, ID, include_nutrition):
    endpoint = "recipes/{ID}/information".format(ID=ID)
    params = {"includeNutrition": include_nutrition}
    return self.make_request(endpoint, params)

  def get_similar_recipes(self, ID, number=1):
    endpoint = "recipes/{ID}/similar".format(ID=ID)
    params = {"number": number}
    return self.make_request(endpoint, params)
    
  def autocomplete_recipe_search(self, query, number=1):
    endpoint = "recipes/autocomplete"
    params = {"query": query, "number": number}
    return self.make_request(endpoint, params)

  def get_analyzed_recipe_instructions(self, ID, stepBreakdown=False):
    endpoint = "recipes/{ID}/analyzedInstructions".format(ID=ID)
    params = {"stepBreakdown": stepBreakdown}
    return self.make_request(endpoint, params)
  
  def ingredient_search(self, query, number, intolerances=None, meta=True):
    endpoint = "food/ingredients/autocomplete"
    params = {"query": query, "number": number,
              "metaInformation": meta, "intolerances": intolerances}
    return self.make_request(endpoint, params)

  def get_ingredient_information(self, ID, amount, unit):
    endpoint = "food/ingredients/{ID}/information".format(ID=ID)
    params = {"amount": amount, "unit": unit}
    return self.make_request(endpoint, params)

  def generate_meal_plan(self, **kwargs):
    endpoint = "mealplanner/generate"
    params = {**kwargs}
    return self.make_request(endpoint, params)

class FoodDatabase(Spoonacular):
  def __init__(self, api_key=os.environ["SPOONACULAR_API_KEY"]):
    Spoonacular.__init__(self, api_key)

  def food_search(self, food, number, intolerances=None):
    results = self.ingredient_search(food, number, intolerances).json()
    search_results = [{"id": food["id"], "name": food["name"], "image": food["image"]} for food in results]
    return search_results

  def food_info(self, food_id, units="g", amount=100):
    results = self.get_ingredient_information(food_id, amount, units).json()
    info = {"name": results["name"], "amount": results["amount"],
            "unit": results["unit"], "image": results["image"],
            "nutrition": results["nutrition"]}
    return info

  def download_food_images(self, search_results, size):
    assert size == 100 or size == 250 or size == 500, "invalid size, valid image sizes are 100, 250 and 500"
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "food_images")
    size = str(size) + "x" + str(size)
    download_url = "https://spoonacular.com/cdn/ingredients_{size}/".format(size=size)
    if not os.path.isdir(path): os.makedirs(path)
    for food in search_results:
      if os.path.isfile(os.path.join(path, food["image"])): continue
      download = requests.get(download_url+food["image"], stream=True)
      if download.ok:
        with open(os.path.join(path, food["image"]), "wb") as img_file:
          download.raw.decode_content = True
          shutil.copyfileobj(download.raw, img_file)
        
class RecipeDatabase(Spoonacular):
  pass



import json
from PyQt5.QtWidgets import (QWidget, QLabel, QPushButton, QHBoxLayout, QVBoxLayout,
                             QFormLayout, QLineEdit, QComboBox, QRadioButton, QGroupBox)
from PyQt5.QtCore import pyqtSignal
from fitness_tracker.user_profile.profile_db import (fetch_user_weight, fetch_goal_weight, fetch_goal,
                                                     fetch_age, fetch_gender, fetch_height, fetch_goal_params,
                                                     update_weight, update_goal, update_goal_parameters,
                                                     update_goal_weight)
from .nutrition_db import update_calorie_goal
from .calorie_goal_calculator import CalorieGoalCalculator

class ChangeWeightDialog(QWidget):
  change_current_weight_signal = pyqtSignal(str)
  change_goal_weight_signal = pyqtSignal(str)
  change_calorie_goal_signal = pyqtSignal(str)

  def __init__(self):
    super().__init__()
    self.user_weight = fetch_user_weight()
    self.goal_weight = fetch_goal_weight()
    goal_parameters = json.loads(fetch_goal_params())
    self.goal = fetch_goal()
    self.activity_level = goal_parameters[0]
    self.weight_per_week = goal_parameters[1]
    self.age = fetch_age()
    self.gender = fetch_gender()
    self.height = fetch_height()
    self.setWindowTitle("Edit weight")
    self.create_layout()
    
  def create_layout(self):
    layout = QFormLayout()

    current_weight_label = QLabel("Current weight")
    self.current_weight_line_edit = QLineEdit()
    self.current_weight_line_edit.setText(self.user_weight)
    
    goal_label = QLabel("Goal")
    goal_layout = QHBoxLayout()
    goal = QGroupBox()
    self.weight_loss_button = QRadioButton("Weight loss")
    self.maintain_weight_button = QRadioButton("Maintain weight")
    self.weight_gain_button = QRadioButton("Weight gain")
    if self.goal == "Weight loss": self.weight_loss_button.setChecked(True)
    elif self.goal == "Maintain weight": self.maintain_weight_button.setChecked(True)
    elif self.goal == "Weight gain": self.weight_gain_button.setChecked(True)
    goal_layout.addWidget(self.weight_loss_button)
    goal_layout.addWidget(self.maintain_weight_button)
    goal_layout.addWidget(self.weight_gain_button)
    goal.setLayout(goal_layout)

    goal_parameters_label = QLabel("Goal parameters")
    goal_parameters_layout = QHBoxLayout()

    activity_level_layout = QVBoxLayout()
    activity_level_label = QLabel("Activity level")
    self.activity_level_cb = QComboBox()
    self.activity_level_cb.addItems(["Sedentary", "Lightly active", "Moderately active", "Very active", "Extra active"])
    self.activity_level_cb.setCurrentText(self.activity_level)
    activity_level_layout.addWidget(activity_level_label)
    activity_level_layout.addWidget(self.activity_level_cb)
    
    weight_per_week_layout = QVBoxLayout()
    weight_per_week_label = QLabel("Weight to lose/gain per week (kg)")
    self.weight_per_week_cb = QComboBox()
    self.weight_per_week_cb.addItems(["0.25", "0.5", "1"])
    self.weight_per_week_cb.setCurrentText(str(self.weight_per_week))
    weight_per_week_layout.addWidget(weight_per_week_label)
    weight_per_week_layout.addWidget(self.weight_per_week_cb)
    
    goal_parameters_layout.addLayout(activity_level_layout)
    goal_parameters_layout.addLayout(weight_per_week_layout)

    goal_weight_label = QLabel("Goal weight")
    self.goal_weight_line_edit = QLineEdit()
    self.goal_weight_line_edit.setText(self.goal_weight)

    save_changes_button = QPushButton("Save changes")
    save_changes_button.clicked.connect(lambda: self.save_changes())
    cancel_button = QPushButton("Cancel")
    cancel_button.clicked.connect(lambda: self.close())

    layout.addRow(current_weight_label, self.current_weight_line_edit)
    layout.addRow(goal_label, goal)
    layout.addRow(goal_parameters_label, goal_parameters_layout)
    layout.addRow(goal_weight_label, self.goal_weight_line_edit)
    layout.addRow(save_changes_button, cancel_button)
    self.setLayout(layout)

  def save_changes(self):
    current_weight = str(float(self.current_weight_line_edit.text()))
    if self.weight_loss_button.isChecked():
      goal = "Weight loss"
    elif self.maintain_weight_button.isChecked():
      goal = "Maintain weight"
    elif self.weight_gain_button.isChecked():
      goal = "Weight gain"
    goal_params = [self.activity_level_cb.currentText(), float(self.weight_per_week_cb.currentText())]
    goal_weight = str(float(self.goal_weight_line_edit.text()))

    try:
      if not current_weight == self.user_weight:
        update_weight(str(float(current_weight)))
        self.user_weight = current_weight
        self.change_current_weight_signal.emit(current_weight)
        self.current_weight_line_edit.setText(current_weight)
      if not goal == self.goal:
        update_goal(goal)
        self.goal = goal
        if goal == "Weight loss":
          self.weight_loss_button.setChecked(True)
        elif goal == "Maintain weight":
          self.maintain_weight_button.setChecked(True)
        elif goal == "Weight gain":
          self.weight_gain_button.setChecked(True)
      if not goal_params[0] == self.activity_level or not goal_params[1] == self.weight_per_week:
        update_goal_parameters(json.dumps(goal_params))
        self.activity_level = goal_params[0]
        self.weight_per_week = goal_params[1]
      if not goal_weight == self.goal_weight:
        update_goal_weight(goal_weight)
        self.goal_weight = goal_weight
        self.change_goal_weight_signal.emit(goal_weight)
        self.goal_weight_line_edit.setText(goal_weight)
      
      calculator = CalorieGoalCalculator(int(self.age), self.gender, float(self.height), float(self.user_weight), goal_params[0], goal, goal_params[1])
      calorie_goal = calculator.calculate_calorie_goal()
      update_calorie_goal(calorie_goal)
      self.change_calorie_goal_signal.emit(str(calorie_goal))
      self.close()
    except ValueError:
      pass



import psycopg2
from psycopg2 import sql
import sqlite3
import json
from fitness_tracker.user_profile.profile_db import logged_in_user_email
from fitness_tracker.config import db_path, db_info

def table_is_empty(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT COUNT (*) FROM 'workouts' WHERE email=?", (email,))
    if cursor.fetchone()[0] == 0: return True
  return False

def create_workouts_table(db_path=db_path):
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    create_table = """
                   CREATE TABLE IF NOT EXISTS
                   'workouts' (
                   email text,
                   workouts text,
                   current_workout_plan text,
                   id integer NOT NULL,
                   PRIMARY KEY(id));
                   """
    cursor.execute(create_table)

def fetch_workouts_table_data(db_path=db_path):
  email = logged_in_user_email(db_path)
  select_workouts = "SELECT workouts FROM workouts WHERE email=%s"
  select_current_workout_plan = "SELECT current_workout_plan FROM workouts WHERE email=%s"

  workouts = None
  current_workout_plan = None

  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute(select_workouts, (email,))
      workouts = cursor.fetchone()[0]

      cursor.execute(select_current_workout_plan, (email,))
      current_workout_plan = cursor.fetchone()[0]
  
  if table_is_empty(db_path):
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute("INSERT INTO 'workouts' (email, workouts, current_workout_plan) VALUES (?, ?, ?)", (email, workouts, current_workout_plan,))

def fetch_workouts(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT workouts FROM 'workouts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_current_workout_plan(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT current_workout_plan FROM 'workouts' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def insert_default_workouts_data(db_path=db_path):
  email = logged_in_user_email(db_path)
  workouts = {}
  current_workout_plan = ""
  default_dict = {"email": email, "workouts": json.dumps(workouts), "current_workout_plan": current_workout_plan}
  try:
    with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
      with conn.cursor() as cursor:
        insert_query = "INSERT INTO workouts ({columns}) VALUES %s"
        columns = sql.SQL(", ").join(sql.Identifier(column) for column in tuple(default_dict.keys()))
        values = tuple(value for value in default_dict.values())
        cursor.execute(sql.SQL(insert_query).format(columns=columns), (values,))

    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      insert_query = "INSERT INTO 'workouts' {columns} VALUES {values}"
      cursor.execute(insert_query.format(columns=tuple(default_dict.keys()), values=tuple(default_dict.values())))
  except psycopg2.errors.UniqueViolation:
    fetch_workouts_table_data(db_path)

def update_workouts(workout_name, new_workout, db_path=db_path):
  email = logged_in_user_email(db_path)
  current_workouts = json.loads(fetch_workouts(db_path))
  current_workouts[workout_name] = new_workout
  new_workout = json.dumps(current_workouts)
  
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE workouts SET workouts=%s WHERE email=%s", (new_workout, email,))
  
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'workouts' SET workouts=? WHERE email=?", (new_workout, email,))

def update_current_workout(workout_name, set_as_current_workout, db_path=db_path):
  if set_as_current_workout == True:
    email = logged_in_user_email(db_path)
    with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
      with conn.cursor() as cursor:
        cursor.execute("UPDATE workouts SET current_workout_plan=%s WHERE email=%s", (workout_name, email,))

    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute("UPDATE 'workouts' SET current_workout_plan=? WHERE email=?", (workout_name, email,))

def delete_workout(workout_name, db_path=db_path):
  email = logged_in_user_email(db_path)
  workouts = json.loads(fetch_workouts(db_path))
  del workouts[workout_name]
  workouts = json.dumps(workouts)
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE workouts SET workouts=%s WHERE email=%s", (workouts, email,))
  
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'workouts' SET workouts=? WHERE email=?", (workouts, email,))



import os
import sys
import psycopg2
from psycopg2 import sql
import sqlite3
from fitness_tracker.common.units_conversion import kg_to_pounds, pounds_to_kg
from fitness_tracker.config import db_info, db_path

def logged_in_user_email(db_path=db_path):
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT email FROM 'users' WHERE logged_in='YES'")
    return cursor.fetchone()[0]

def fetch_local_user_data():
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[2:]

def fetch_username(db_path=db_path):
  try:
    email = logged_in_user_email()
    with sqlite3.connect(db_path) as conn:
      cursor = conn.cursor()
      cursor.execute("SELECT name FROM 'users' WHERE email=?", (email,))
      return cursor.fetchone()[0]
  except sqlite3.OperationalError: # titlebar will raise this error if users table doesn't exist
    pass

def fetch_units(db_path=db_path):
  email = logged_in_user_email(db_path)
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT units FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_user_weight(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT weight FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_age(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT age FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_gender(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT gender FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_height(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT height FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_goal_weight(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT goalweight FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_goal_params(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT goalparams FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def fetch_goal(db_path=db_path):
  email = logged_in_user_email()
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("SELECT goal FROM 'users' WHERE email=?", (email,))
    return cursor.fetchone()[0]

def set_weight(user_data):
  # 4th column of user table is currently weight
  weight = user_data[4]
  if "metric" in user_data: return " ".join([str(weight), "kg"])
  elif "imperial" in user_data: return " ".join([str(weight), "lb"])
  
def convert_weight(current_units, weight):
  if current_units == "metric":
    return kg_to_pounds(float(weight))
  elif current_units == "imperial":
    return pounds_to_kg(float(weight))

def update_weight(weight, db_path=db_path):
  email = logged_in_user_email()
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE users SET weight=%s WHERE email=%s", (str(weight), email,))
      
  
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor()
    cursor.execute("UPDATE 'users' SET weight=? WHERE email=?", (str(weight), email,))
  
def update_goal(goal, db_path=db_path):
  email = logged_in_user_email()
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE users SET goal=%s WHERE email=%s", (goal, email,))

  with sqlite3.connect(db_path) as conn: 
    cursor = conn.cursor()
    cursor.execute("UPDATE 'users' SET goal=? WHERE email=?", (goal, email,))

def update_goal_parameters(goal_params, db_path=db_path):
  email = logged_in_user_email()
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE users SET goalparams=%s WHERE email=%s", (goal_params, email,))

  with sqlite3.connect(db_path) as conn: 
    cursor = conn.cursor()
    cursor.execute("UPDATE 'users' SET goalparams=? WHERE email=?", (goal_params, email,))

def update_goal_weight(goal_weight, db_path=db_path):
  email = logged_in_user_email()
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE users SET goalweight=%s WHERE email=%s", (goal_weight, email,))

  with sqlite3.connect(db_path) as conn: 
    cursor = conn.cursor()
    cursor.execute("UPDATE 'users' SET goalweight=? WHERE email=?", (goal_weight, email,))

def update_calorie_goal(calorie_goal, db_path=db_path):
  email = logged_in_user_email()
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute("UPDATE nutrition SET calorie_goal=%s WHERE email=%s", (calorie_goal, email,))

  with sqlite3.connect(db_path) as conn: 
    cursor = conn.cursor()
    cursor.execute("UPDATE nutrition SET calorie_goal=? WHERE email=?", (calorie_goal, email,))

def update_units():
  email = logged_in_user_email()
  current_units = None
  with sqlite3.connect(db_path) as conn:
    cursor = conn.cursor() 
    set_units_imperial = "UPDATE 'users' SET units='imperial' WHERE email=?"
    set_units_metric = "UPDATE 'users' SET units='metric' WHERE email=?"
    cursor.execute("SELECT units FROM 'users' WHERE email=?", (email,))
    current_units = cursor.fetchone()[0]
    update_units = set_units_imperial if current_units == "metric" else set_units_metric
    cursor.execute(update_units, (email,))
 
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      set_units_imperial = "UPDATE users SET units='imperial' WHERE email=%s"
      set_units_metric = "UPDATE users SET units='metric' WHERE email=%s"
      update_units = set_units_imperial if current_units == "metric" else set_units_metric
      cursor.execute(update_units, (email,))



import requests
import os
import shutil

class WorkoutsAPI:
  def __init__(self):
    self.base_api_url = "https://wger.de/api/v2/"
    self.exercises_url = self.base_api_url + "exercise/"
    self.images_url = self.base_api_url + "exerciseimage/"
    self.exercise_info_url = self.base_api_url + "exerciseinfo/{ID}"
    self.language = 2 # English
    # exercise category: (exercise category id, [muscle group id])
    self.exercise_categories = {"Biceps": (8, [1, 13]), "Shoulders": (13, [2]),
                                "Chest": (11, [4]), "Triceps": (8, [5]),
                                "Legs": (9, [10, 11, 8]), "Calves": (14, [15, 7]),
                                "Back": (12, [12, 9]), "Core": (10, [6, 14, 3])}

    self.equipment = {"Barbell": 1, "EZ Bar": 2, "Dumbbell": 3, "Gym mat": 4,
                      "Swiss ball": 4, "Pull-up bar": 6, "Bodyweight": 7,
                      "Bench": 8, "Incline bench": 9, "Kettlebell": 10}
     
    self.muscle_category = {1: "Biceps brachii", 2: "Anterior deltoid",
                            3: "Serratus anterior", 4: "Pectoralis major",
                            5: "Triceps brachii", 6: "Rectus abdominis",
                            7: "Gastrocnemius", 8: "Gluteus maximus",
                            9: "Trapezius", 10: "Quadriceps femoris",
                            11: "Biceps femoris", 12: "Latissimus dorsi",
                            13: "Brachialis", 14: "Obliquus externus abdominis",
                            15: "Soleus"}
    
  def parse_exercises(self, fetched_exercises):
    parsed = []
    for exercise in fetched_exercises["results"]:
      parsed_exercise = {"exercise_id": exercise["id"], "category": exercise["category"],
                         "description": exercise["description"], "name": exercise["name"],
                         "equipment": exercise["equipment"]}
      if len(exercise["muscles"]) > 0 or len(exercise["muscles_secondary"]) > 0:
        primary, secondary = [], []
        for muscle_category in self.muscle_category:
          if muscle_category in exercise["muscles"]: primary.append(self.muscle_category[muscle_category])
          if muscle_category in exercise["muscles_secondary"]: secondary.append(self.muscle_category[muscle_category])
        parsed_exercise["muscles_primary"] = primary
        parsed_exercise["muscles_secondary"] = secondary
      if len(exercise["equipment"]) > 0:
        parsed_equipment = []
        for equipment in self.equipment:
          if self.equipment[equipment] in exercise["equipment"]: parsed_equipment.append(equipment)
        parsed_exercise["equipment"] = parsed_equipment
      parsed_exercise["image"] = self.fetch_images([parsed_exercise["exercise_id"]], single_image=True)
      fetch_comment = requests.get(url=self.exercise_info_url.format(ID=parsed_exercise["exercise_id"])).json()
      if len(fetch_comment["comments"]) > 0:
        parsed_exercise["comment"] = fetch_comment["comments"][0]["comment"]
      else:
        parsed_exercise["comment"] = []
      parsed.append(parsed_exercise)
    return parsed
   
  def fetch_exercises(self, exercise_category, equipment=None, limit=5, **kwargs):
    params = {"language": self.language, "muscles": self.exercise_categories[exercise_category][1],
              "category": self.exercise_categories[exercise_category][0], "limit": limit,
              **kwargs}
    if equipment != None:
      parse_equipment = []
      for i in equipment:
        parse_equipment.append(self.equipment[i])
      params["equipment"] = parse_equipment 
    fetched_exercises = requests.get(url=self.exercises_url, params=params).json()
    return self.parse_exercises(fetched_exercises)
  
  def fetch_images(self, exercises, single_image=False):
    urls = []
    params = {"is_main": True, "limit": 204}
    images = requests.get(self.images_url, params=params).json()
    if single_image: exercise_ids = exercises
    else: exercise_ids = [exercise["exercise_id"] for exercise in exercises]
    for exercise_id in exercise_ids:
      for image in images["results"]:
        if image["exercise"] == exercise_id:
          urls.append(image["image"])
    return urls

  def download_exercise_images(self, exercises):
    urls = self.fetch_images(exercises) 
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "exercises", "exercise_images")
    if not os.path.isdir(path): os.makedirs(path)
    for image in urls:
      image_name = os.path.basename(image)
      if os.path.isfile(os.path.join(path, image_name)): continue
      download = requests.get(image, stream=True)
      if download.ok:
        with open(os.path.join(path, image_name), "wb") as img_file:
          download.raw.deconde_content = True
          shutil.copyfileobj(download.raw, img_file)



import hashlib
import psycopg2
import sqlite3
from fitness_tracker.config import db_info, db_path

def check_password(email, password):
  status = True
  query = "SELECT * FROM users WHERE email=%s"
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
    with conn.cursor() as cursor:
      cursor.execute(query, (email,))
      database_password = cursor.fetchone()[1]
      if not hashlib.sha256(password.encode('UTF-8')).hexdigest() == database_password: status = False
  return status

def fetch_user_info(email, password, db_path=db_path):
  columns = None
  user_info = None
  with psycopg2.connect(host=db_info["host"], port=db_info["port"], database=db_info["database"],
                        user=db_info["user"], password=db_info["password"]) as conn:
      with conn.cursor() as cursor:
        # this can be changed to fetch something else later on
        cursor.execute(("SELECT * FROM users WHERE email='{email}'").format(email=email))
        user_info = cursor.fetchone()[:-1]
        # fetch all column names
        # change this in the future
        query = """
                SELECT column_name FROM information_schema.columns 
                WHERE table_name = 'users';
                """
        cursor.execute(query)
        columns = tuple(value[0] for value in cursor.fetchall() if not value[0] == 'id')
  
  with sqlite3.connect(db_path) as conn:
    table_exists = "SELECT COUNT(*) FROM sqlite_master WHERE type='table' AND name='users';"
    cursor = conn.cursor()
    cursor.execute(table_exists)
    if cursor.fetchone()[0] == 0: # table doesn't exist
      create_table = """
                     CREATE TABLE "users" (
                     email text NOT NULL,
                     password text NOT NULL,
                     name text NOT NULL,
                     age text NOT NULL,
                     gender text NOT NULL,
                     units text NOT NULL,
                     weight text NOT NULL,
                     height text NOT NULL,
                     goal text NOT NULL,
                     goalparams text NOT NULL, 
                     goalweight text NOT NULL,
                     logged_in text,
                     ID integer NOT NULL,
                     PRIMARY KEY (ID));
                     """
      cursor.execute(create_table)
      insert_values = "INSERT INTO 'users' {columns} VALUES {values}"
      cursor.execute(insert_values.format(columns=columns, values=user_info))
      set_logged_in = "UPDATE users SET logged_in='YES'"
      cursor.execute(set_logged_in)



from fitness_tracker.common.units_conversion import pounds_to_kg, imperial_to_metric_height
from fitness_tracker.user_profile.profile_db import fetch_units

class CalorieGoalCalculator:
  def __init__(self, age, gender, height, weight, activity_level, goal, weight_per_week):
    self.units = fetch_units()
    self.age = age
    self.gender = gender
    activity_factors = {"Maintain": 1, "Sedentary": 1.2, "Lightly active": 1.375,
                        "Moderately active": 1.550, "Very active": 1.725,
                        "Extra active": 1.9}
    self.activity_level = activity_factors[activity_level]
    if self.units == "imperial":
      self.height = imperial_to_metric_height(height["feet"], height["inches"])
      self.weight = pounds_to_kg(weight)
    else:
      self.height = height
      self.weight = weight
    self.goal = goal
    self.weight_per_week = weight_per_week
  
  def calculate_calorie_goal(self):
    if self.goal == "Maintain weight": return self.calculate_BMR()
    elif self.goal == "Weight loss": return self.calculate_weight_loss_goal(self.weight_per_week)
    elif self.goal == "Weight gain": return self.calculate_weight_gain_goal(self.weight_per_week)

  def calculate_BMR(self):
    """
    The Basal Metabolic Rate - amount of energy expended while at rest in a neutrally temperate environment, 
    and in a post-absorptive state (meaning that the digestive system is inactive, which requires about 12 hours of fasting).
    """
    equ = 10*self.weight+6.25*self.height-5*self.age # Mifflin-St Jeor Equation
    if self.gender == "male": equ += 5
    elif self.gender == "female": equ -= 161
    return round(equ*self.activity_level)

  def calculate_weight_loss_goal(self, weight_per_week):
    assert weight_per_week == 0.25 or weight_per_week == 0.5 or weight_per_week == 1 # kg
    calories_per_day = self.calculate_BMR()
    if weight_per_week == 0.25:
      return round(calories_per_day*0.89)
    elif weight_per_week == 0.5:
      return round(calories_per_day*0.78)
    elif weight_per_week == 1:
      return round(calories_per_day*0.56)
  
  def calculate_weight_gain_goal(self, weight_per_week):
    assert weight_per_week == 0.25 or weight_per_week == 0.5 or weight_per_week == 1 # kg
    calories_per_day = self.calculate_BMR()
    if weight_per_week == 0.25:
      return round(calories_per_day*1.11)
    elif weight_per_week == 0.5:
      return round(calories_per_day*1.22)
    elif weight_per_week == 1:
      return round(calories_per_day*1.44)



import os
from functools import partial
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QGridLayout, QSpacerItem, QSizePolicy
from PyQt5.QtGui import QCursor, QIcon, QFont, QPixmap
from PyQt5.QtCore import Qt, QSize, pyqtSignal

path = os.path.abspath(os.path.dirname(__file__))

icons_path = os.path.join(path, "icons")
images_path = os.path.join(path, "food_images")

class SearchResultsPanel(QWidget):
  return_to_food_db_signal = pyqtSignal(str)
  food_panel_signal = pyqtSignal(int)

  def __init__(self, parent, search_results):
    super().__init__(parent)
    self.search = search_results[1]
    self.search_results = search_results[0]
    self.create_panel()
    self.setStyleSheet("QLabel{color:white;}")

  def create_panel(self):
    grid = QGridLayout()
    grid.addLayout(self.create_results_summary(), 0, 0)
    grid.addLayout(self.create_results(), 1, 0)
    self.setLayout(grid)

  def create_results_summary(self):
    summary_layout = QHBoxLayout()
    back_button = QPushButton()
    back_button.setFlat(True)
    back_button.setIcon(QIcon(os.path.join(icons_path, "back.png")))
    back_button.setIconSize(QSize(24, 24))
    back_button.setCursor(QCursor(Qt.PointingHandCursor))
    back_button.clicked.connect(lambda: self.return_to_food_db_signal.emit("Return to FoodDatabase"))

    results_label = QLabel('Found %s results for "%s"' % (len(self.search_results), self.search))
    results_label.setFont(QFont("Ariel", 15))
    
    spacer = QSpacerItem(40, 20, QSizePolicy.Expanding)
    
    search_bar = QLineEdit()
    search_bar.setStyleSheet("background-color:white;")
    search_bar.setPlaceholderText("Search")
    
    search_button = QPushButton("Search")
    search_button.setStyleSheet("background-color: #440D0F;color:white;")
    search_button.setCursor(QCursor(Qt.PointingHandCursor))

    summary_layout.addWidget(back_button)
    summary_layout.addWidget(results_label)
    summary_layout.addSpacerItem(spacer)
    summary_layout.addWidget(search_bar)
    summary_layout.addWidget(search_button)

    return summary_layout

  def create_results(self):
    results_grid = QGridLayout()
    
    result_layouts = [None] * len(self.search_results)
    result_images = [None] * len(self.search_results)
    result_labels = [None] * len(self.search_results)
    
    i = 0
    row = 0
    for result in self.search_results:
      result_layouts[i] = QVBoxLayout()
      result_images[i] = QPushButton()
      result_labels[i] = QLabel()

      result_images[i].setProperty("food_id", result["id"])
      result_images[i].clicked.connect(partial(self.food_panel_signal.emit, result_images[i].property("food_id")))
      result_images[i].setFlat(True)
      result_images[i].setIcon(QIcon(os.path.join(images_path, result["image"])))
      result_images[i].setIconSize(QSize(250, 200))
      result_images[i].setCursor(QCursor(Qt.PointingHandCursor))
      result_labels[i].setText(result["name"].capitalize())
      result_labels[i].setAlignment(Qt.AlignCenter)

      result_layouts[i].addWidget(result_images[i])
      result_layouts[i].addWidget(result_labels[i])
      results_grid.addLayout(result_layouts[i], row, i)
      i += 1
      if i == 3: 
        i = 0
        row += 1
    return results_grid 
